<?php
/**
 *   Copyright (c) 2018, MonkeyData s.r.o.
 *
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *    * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *    * Neither the name of the <organization> nor the
 *   names of its contributors may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *   @author    MonkeyData
 *   @copyright 2018 MonkeyData s.r.o.
 *   @license   MIT
 */

namespace MonkeyData\Inside\App\Installation;

use Monkey\ImportEshopApi\Base\ASecurityHandler;

/**
 * Class PrestaShopSecurityHandler
 */
class PrestaShopSecurityHandler
{
    /**
     * @var \Monkey\PluginsPrestashop15XApi\LocalhostSecurityHandler
     */
    private $securityHandler;

    /**
     * PrestaShopSecurityHandler constructor.
     * @throws \Exception
     */
    public function __construct()
    {
        $factory = new \Monkey\ImportEshopApi\Base\SecurityHandlerFactory();
        $factory->registerSecurityHandler(
            \Monkey\PluginsPrestashop17XApi\LocalhostSecurityHandler::VERSION
        );
        $factory->registerSecurityHandler(
            \Monkey\PluginsPrestashop16XApi\LocalhostSecurityHandler::VERSION
        );
        $factory->registerSecurityHandler(
            \Monkey\PluginsPrestashop15XApi\LocalhostSecurityHandler::VERSION
        );

        $securityHandler = $factory->build(_PS_VERSION_);
        $this->setSecurityHandler($securityHandler);
    }

    /**
     * @param $database
     * @param $engine
     * @return array
     */
    public function getTableCreates($database, $engine)
    {
        $securityHandler = $this->getSecurityHandler();
        return $securityHandler::getTableCreates($database, $engine);
    }

    /**
     * @param $database
     * @return array
     */
    public function getTableDrops($database)
    {
        $securityHandler = $this->getSecurityHandler();
        return $securityHandler::getTableDrops($database);
    }

    /**
     * @param $token
     */
    public function storeTokenToLocalStorage($token)
    {
        $this->getSecurityHandler()->storeTokenToLocalStorage($token);
    }

    /**
     * @return \Monkey\PluginsPrestashop15XApi\LocalhostSecurityHandler
     */
    protected function getSecurityHandler()
    {
        return $this->securityHandler;
    }

    /**
     * @param ASecurityHandler $securityHandler
     * @return $this
     */
    private function setSecurityHandler(ASecurityHandler $securityHandler)
    {
        $this->securityHandler = $securityHandler;
        return $this;
    }
}
