<?php
/**
 *   Copyright (c) 2018, MonkeyData s.r.o.
 *
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *    * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *    * Neither the name of the <organization> nor the
 *   names of its contributors may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *   @author    MonkeyData
 *   @copyright 2018 MonkeyData s.r.o.
 *   @license   MIT
 */

/**
 * Class AdminMonkeyDataInsideProductStatisticsController
 */
class AdminMonkeyDataInsideProductStatisticsController extends ModuleAdminController
{
    public $template = 'adminMonkeyData.tpl';

    public function initContent()
    {
        parent::initContent();

        require_once dirname(__FILE__) . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..'
            . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php';

        $model = new \MonkeyData\Inside\App\PrestaShopModel();
        $config = new \MonkeyData\Inside\App\PrestaShopConfig();

        $insideController = new \MonkeyData\Inside\App\PrestaShopInsideController($model, $config);

        $insideRequest = new \Monkey\InsideBootJs\Base\InsideRequest();

        $authCode = Tools::getValue(\Monkey\InsideAuth\Interfaces\IInsideAuth::FIELD_NAME_CODE);

        if (!empty($authCode)) {
            $insideRequest->setAuthCode($authCode);
        }

        try {
            $insideBootJs = $insideController->handle($insideRequest);
            $bootJS = $insideBootJs->generateBootJS();
            $anchor = $insideBootJs->generateAnchorElement();
        } catch (\Monkey\InsideAuth\Exceptions\InsideAuthException $exception) {
            \Monkey\InsideSentry\InsideSentry::error($exception);
            $bootJS = 'Communication error, please try again later';
            $anchor = 'Or try to contact MonkeyData support.';
        }

        // template variables
        $this->context->smarty->assign(array ('bootJS' => $bootJS, 'anchor' => $anchor));
    }
}
