<?php
/**
 *   Copyright (c) 2018, MonkeyData s.r.o.
 *
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *    * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *    * Neither the name of the <organization> nor the
 *   names of its contributors may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *   DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 *   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *   @author    MonkeyData
 *   @copyright 2018 MonkeyData s.r.o.
 *   @license   MIT
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class Productstatistics extends Module
{
    public function __construct()
    {
        $this->name = 'productstatistics';
        $this->tab = 'analytics_stats';
        $this->version = '1.0.0';
        $this->author = 'MonkeyData';
        $this->credits = 'Module authors are MonkeyData.';
        $this->need_instance = 0;
        $this->bootstrap = true;
        $this->is_configurable = 0;
        $this->module_key = '2ab84efd4354ca01fef06188cb83c3d1';
        $this->ps_versions_compliancy = array ('min' => '1.5', 'max' => '1.7');
        parent::__construct();
        $this->displayName = $this->l('Product Statistics');
        $this->description = $this->l(
            "Adds graphs showing all key product related stats to give you a clear oversight of your store's potential."
        );
        $this->confirmUninstall = $this->l("Do you really want to uninstall {$this->displayName}?");
    }

    public function install()
    {
        if (!parent::install()
            || !$this->registerHooks()
            || !$this->pluginInstallationHandle()) {
            return false;
        }

        return true;
    }

    private function registerHooks()
    {
        $this->registerHook('displayBackOfficeHeader');
        return true;
    }

    public function uninstall()
    {
        if (!parent::uninstall()
            || !$this->unregisterHooks()
            || !$this->pluginUninstallationHandle()) {
            return false;
        }

        return true;
    }

    private function unregisterHooks()
    {
        $this->unregisterHook('displayBackOfficeHeader');
        return true;
    }

    /**
     * @return bool
     */
    private function pluginInstallationHandle()
    {
        require_once dirname(__FILE__) . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php';

        try {
            $securityHandler = new \MonkeyData\Inside\App\Installation\PrestaShopSecurityHandler();
        } catch (Exception $exception) {
            return false;
        }

        $errorStack = new MonkeyData\Inside\App\ErrorStack\PrestaShopErrorStack($this->context->controller);

        $handle = new \MonkeyData\Inside\App\Installation\PluginInstallationHandle($securityHandler, $errorStack);

        if ($handle->install() === false) {
            return false;
        }

        return true;
    }

    /**
     * @return bool
     */
    private function pluginUninstallationHandle()
    {
        require_once dirname(__FILE__) . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php';

        try {
            $securityHandler = new \MonkeyData\Inside\App\Installation\PrestaShopSecurityHandler();
        } catch (Exception $exception) {
            return false;
        }

        $errorStack = new MonkeyData\Inside\App\ErrorStack\PrestaShopErrorStack($this->context->controller);

        $handle = new \MonkeyData\Inside\App\Installation\PluginInstallationHandle($securityHandler, $errorStack);

        if ($handle->uninstall() === false) {
            return false;
        }

        return true;
    }

    public function hookDisplayBackOfficeHeader()
    {
        $this->context->controller->addCss($this->_path.'views/css/tab.css');
    }

    public function getContent()
    {
        require_once dirname(__FILE__) . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php';

        // consider putting this to monkey package
        $this->context->controller->addJS($this->_path . '/vendor/monkey/inside-boot-js/assets/js/boot-js-core.js');
        $this->context->controller->addJS($this->_path . '/vendor/monkey/inside-boot-js/assets/js/boot-js-grub.js');

        $token = Tools::getAdminTokenLite('AdminModules');
        $pluginController = \MonkeyData\Inside\CurrentPluginConfiguration::PLUGIN_TAB_CONTROLLER;
        $pluginTabName = \MonkeyData\Inside\CurrentPluginConfiguration::PLUGIN_TAB_NAME;
        $model = new \MonkeyData\Inside\App\PrestaShopModel();
        $hasSecretToken = true;
        if ($model->getSecretToken() === null) {
            $hasSecretToken = false;
        }

        $reconnectUrl = AdminControllerCore::$currentIndex.'&configure='.$this->name.'&token='.$token.'&reconnect=true';

        $variables = array (
            'tabName' => $pluginTabName,
            'reconnected' => false,
            'reconnectUrl' => $reconnectUrl,
            'controllerUrl' => $this->context->link->getAdminLink($pluginController),
            'monkeyDataPluginsUrl' => $this->getConfigurationUrl(),
            'monkeyDataPluginsElementName' => 'MonkeyDataInsideConfiguration',
            'hasSecretToken' => $hasSecretToken
        );

        if (Tools::getValue('reconnect') !== false) {
            $model = new \MonkeyData\Inside\App\PrestaShopModel();
            $config = new \MonkeyData\Inside\App\PrestaShopConfig();
            $stack = new MonkeyData\Inside\App\ErrorStack\PrestaShopErrorStack($this->context->controller);
            try {
                $securityHandler = new \MonkeyData\Inside\App\Installation\PrestaShopSecurityHandler();

                $handle = new \MonkeyData\Inside\App\Installation\PluginInstallationHandle($securityHandler, $stack);
                $handle->setupApiAccessToken();

                $remoteInstaller = new \MonkeyData\Inside\App\PrestaShopRemoteInstallController($model, $config);
                $remoteInstaller->install();

                $variables['reconnected'] = true;
            } catch (Exception $exception) {
                \Monkey\InsideSentry\InsideSentry::error($exception);
                $stack->add('Error connection online store');
            }
        }

        $this->smarty->assign($variables);

        if (version_compare(_PS_VERSION_, '1.6', '<')) {
            $this->context->controller->addCSS($this->_path . '/views/css/configuration_1_5.css');
        }
        return $this->display(__FILE__, '/views/templates/admin/configuration.tpl');
    }

    /**
     * @return string
     */
    private function getConfigurationUrl()
    {
        $config = new \MonkeyData\Inside\App\PrestaShopConfig();
        $parts = $config->getInsideUrlParts();
        $parts[] = 'js';
        $parts[] = 'config';
        return implode('/', $parts);
    }
}
