<?php

namespace Monkey\ImportEshopApi\Base;

use Monkey\ImportEshopApi\Exceptions\MissingTokenOnClientSideException;
use Monkey\ImportEshopApi\Exceptions\WrongTokenException;
use Monkey\ImportEshopApi\Interfaces\ISecurityHandler;

/**
 * Class ASecurityHandler
 * @package Monkey\ImportEshopApi\Base
 */
abstract class ASecurityHandler implements ISecurityHandler
{

    /**
     * @param $token
     * @param $token
     * @throws MissingTokenOnClientSideException
     * @throws WrongTokenException
     */
    public function checkToken($token) {
        // we expect the token is checked to be string before this function called -> verify! - solved in prepareToken function
        try{
            $knownToken = $this->loadTokenFromLocalStorage();
        }catch(\Exception $exception){
            throw new MissingTokenOnClientSideException("loading from DB");
        }


        if (!is_string($knownToken)) {
            //if loaded string is not string (e.g. could be null)
            throw new MissingTokenOnClientSideException("token isn't string");
        }

        if (strlen($knownToken) === 0) {
            //empty token
            throw new MissingTokenOnClientSideException("token is empty");
        }

        if (strcmp($knownToken,$this->prepareToken($token)) !== 0) {
            usleep(500000); // sleep 0.5 s
            throw new WrongTokenException();
        }
    }

    /**
     * @return string|null
     */
    abstract protected function loadTokenFromLocalStorage();

    /**
     * @return string|null
     */
    abstract protected function storeTokenToLocalStorage($token);

    /**
     * @param $token
     * @return mixed
     */
    abstract protected function prepareToken($token);

}