<?php

namespace Monkey\ImportEshopApi\Base;

use Monkey\ImportEshopApi\Exceptions\UnknownEndpointException;
use Monkey\ImportEshopDataObjects\Interfaces\IRequest;
use Monkey\ImportEshopDataObjects\Interfaces\IResponse;

/**
 * Class MinerFactory
 * @package Monkey\ImportEshopApi\Base
 */
class MinerFactory {
    /**
     * @var string[]
     */
    private $miners;

    /**
     * @return string[]
     */
    public function getMiners() {
        return $this->miners;
    }

    /**
     * @param ADataMiner $miner
     */
    public function addMiner($miner) {
        $this->miners[$miner->getEndpoint()] = get_class($miner);
    }

    /**
     * @param string $endpoint
     * @param IRequest $request
     * @param IResponse $response
     * @param ADefaultEshopSetting $defaultEshopSetting
     * @return ADataMiner
     * @throws UnknownEndpointException
     */
    public function getMiner($endpoint, $request, $response, $defaultEshopSetting) {

        if (array_key_exists($endpoint, $this->miners)) {

            $minerClassName = $this->miners[$endpoint];

            return new $minerClassName($request, $response, $defaultEshopSetting);
        }
        throw new UnknownEndpointException($endpoint);
    }
}