<?php

namespace Monkey\ImportEshopApi\Base;

use Exception;

/**
 * Class SecurityHandlerFactory
 * @package Monkey\ImportEshopApi\Base
 */
class SecurityHandlerFactory {

    /**
     * @var array
     */
    private $handlers = array();

    /**
     * @param $storeVersion
     * @return ASecurityHandler
     * @throws Exception
     */
    public function build($storeVersion){
        $versionArray = explode(".", $storeVersion);

        $currentVersionName = $versionArray[0].$versionArray[1].$versionArray[2];
        foreach ($this->getHandlers() as $handlerName){
            if(strpos($handlerName, $currentVersionName)!==false){
                return new $handlerName;
            }
        }

        $baseVersionName = $versionArray[0].$versionArray[1]."X";
        foreach ($this->getHandlers() as $handlerName){
            if(strpos($handlerName, $baseVersionName)!==false){
                return new $handlerName;
            }
        }

        throw new Exception("Missing SecurityHandler");
    }

    /**
     * @return array
     */
    public function getHandlers(){
        return $this->handlers;
    }

    /**
     * @param string $securityHandlerClass
     * @return $this
     */
    public function registerSecurityHandler($securityHandlerClass){
        $this->handlers[$securityHandlerClass] = $securityHandlerClass;
        return $this;
    }


}