<?php

namespace Monkey\ImportEshopApi\Endpoint;

use Monkey\ImportEshopApi\Base\ADataMiner;
use Monkey\ImportEshopApi\Base\ADefaultEshopSetting;
use Monkey\ImportEshopApi\Exceptions\WrongRequestObjectException;
use Monkey\ImportEshopApi\Exceptions\WrongResponseObjectException;
use Monkey\ImportEshopDataObjects\Interfaces\IRequest;
use Monkey\ImportEshopDataObjects\Interfaces\IResponse;
use Monkey\ImportEshopDataObjects\Request\AttributesRequest;
use Monkey\ImportEshopDataObjects\Response\AttributesResponse;

/**
 * Class AAttributes
 * @package Monkey\ImportEshopApi\Endpoint
 */
abstract class AAttributes extends ADataMiner
{

    const ENDPOINT = "attributes";

    /**
     * AOrders constructor.
     * @param IRequest $request
     * @param IResponse $response
     * @param ADefaultEshopSetting $defaultEshopSetting
     */
    public function __construct($request,$response, $defaultEshopSetting){
        parent::__construct($request, $response,self::ENDPOINT,$defaultEshopSetting);
    }

    /**
     * @return AttributesRequest
     */
    public function getRequest(){
        return $this->isCorrectRequest(parent::getRequest());
    }

    /**
     * @return AttributesResponse
     */
    public function getResponse(){
        return $this->isCorrectResponse(parent::getResponse());
    }

    /**
     * @param IRequest $request
     * @return AttributesRequest
     * @throws WrongRequestObjectException
     */
    private function isCorrectRequest(IRequest $request) {
        if ($request instanceof AttributesRequest) return $request;
        throw new WrongRequestObjectException(get_class($request),get_class(new AttributesRequest()));
    }

    /**
     * @param IResponse $response
     * @return AttributesResponse
     * @throws WrongResponseObjectException
     */
    private function isCorrectResponse(IResponse $response) {
        if ($response instanceof AttributesResponse) return $response;
        throw new WrongResponseObjectException(get_class($response),get_class(new AttributesResponse()));
    }


}