<?php

namespace Monkey\ImportEshopApi\Endpoint;

use Monkey\ImportEshopApi\Base\ADataMiner;
use Monkey\ImportEshopApi\Base\ADefaultEshopSetting;
use Monkey\ImportEshopApi\Exceptions\WrongRequestObjectException;
use Monkey\ImportEshopApi\Exceptions\WrongResponseObjectException;
use Monkey\ImportEshopDataObjects\Interfaces\IRequest;
use Monkey\ImportEshopDataObjects\Interfaces\IResponse;
use Monkey\ImportEshopDataObjects\Request\CategoriesRequest;
use Monkey\ImportEshopDataObjects\Response\CategoriesResponse;

/**
 * Class ACategories
 * @package Monkey\ImportEshopApi\Endpoint
 */
abstract class ACategories extends ADataMiner
{

    const ENDPOINT = "categories";

    /**
     * AOrders constructor.
     * @param IRequest $request
     * @param IResponse $response
     * @param ADefaultEshopSetting $defaultEshopSetting
     */
    public function __construct($request,$response, $defaultEshopSetting){
        parent::__construct($request, $response,self::ENDPOINT,$defaultEshopSetting);
    }

    /**
     * @return CategoriesRequest
     */
    public function getRequest(){
        return $this->isCorrectRequest(parent::getRequest());
    }

    /**
     * @return CategoriesResponse
     */
    public function getResponse(){
        return $this->isCorrectResponse(parent::getResponse());
    }

    /**
     * @param IRequest $request
     * @return CategoriesRequest
     * @throws WrongRequestObjectException
     */
    private function isCorrectRequest(IRequest $request) {
        if ($request instanceof CategoriesRequest) return $request;
        throw new WrongRequestObjectException(get_class($request),get_class(new CategoriesRequest()));
    }

    /**
     * @param IResponse $response
     * @return CategoriesResponse
     * @throws WrongResponseObjectException
     */
    private function isCorrectResponse(IResponse $response) {
        if ($response instanceof CategoriesResponse) return $response;
        throw new WrongResponseObjectException(get_class($response),get_class(new CategoriesResponse()));
    }


}