<?php

namespace Monkey\ImportEshopApi\Endpoint;

use Monkey\ImportEshopApi\Base\ADataMiner;
use Monkey\ImportEshopApi\Base\ADefaultEshopSetting;
use Monkey\ImportEshopApi\Exceptions\WrongRequestObjectException;
use Monkey\ImportEshopApi\Exceptions\WrongResponseObjectException;
use Monkey\ImportEshopDataObjects\Interfaces\IRequest;
use Monkey\ImportEshopDataObjects\Interfaces\IResponse;
use Monkey\ImportEshopDataObjects\Request\CouponsRequest;
use Monkey\ImportEshopDataObjects\Response\CouponsResponse;

/**
 * Class ACoupons
 * @package Monkey\ImportEshopApi\Endpoint
 */
abstract class ACoupons extends ADataMiner
{

    const ENDPOINT = "coupons";

    /**
     * AOrders constructor.
     * @param IRequest $request
     * @param IResponse $response
     * @param ADefaultEshopSetting $defaultEshopSetting
     */
    public function __construct($request,$response, $defaultEshopSetting){
        parent::__construct($request, $response,self::ENDPOINT,$defaultEshopSetting);
    }

    /**
     * @return CouponsRequest
     */
    public function getRequest(){
        return $this->isCorrectRequest(parent::getRequest());
    }

    /**
     * @return CouponsResponse
     */
    public function getResponse(){
        return $this->isCorrectResponse(parent::getResponse());
    }

    /**
     * @param IRequest $request
     * @return CouponsRequest
     * @throws WrongRequestObjectException
     */
    private function isCorrectRequest(IRequest $request) {
        if ($request instanceof CouponsRequest) return $request;
        throw new WrongRequestObjectException(get_class($request),get_class(new CouponsRequest()));
    }

    /**
     * @param IResponse $response
     * @return CouponsResponse
     * @throws WrongResponseObjectException
     */
    private function isCorrectResponse(IResponse $response) {
        if ($response instanceof CouponsResponse) return $response;
        throw new WrongResponseObjectException(get_class($response),get_class(new CouponsResponse()));
    }


}