<?php

namespace Monkey\ImportEshopApi\Endpoint;

use Monkey\ImportEshopApi\Base\ADataMiner;
use Monkey\ImportEshopApi\Base\ADefaultEshopSetting;
use Monkey\ImportEshopApi\Exceptions\WrongRequestObjectException;
use Monkey\ImportEshopApi\Exceptions\WrongResponseObjectException;
use Monkey\ImportEshopDataObjects\Interfaces\IRequest;
use Monkey\ImportEshopDataObjects\Interfaces\IResponse;
use Monkey\ImportEshopDataObjects\Request\CustomersRequest;
use Monkey\ImportEshopDataObjects\Response\CustomersResponse;

/**
 * Class ACustomers
 * @package Monkey\ImportEshopApi\Endpoint
 */
abstract class ACustomers extends ADataMiner
{

    const ENDPOINT = "customers";

    /**
     * AOrders constructor.
     * @param IRequest $request
     * @param IResponse $response
     * @param ADefaultEshopSetting $defaultEshopSetting
     */
    public function __construct($request,$response, $defaultEshopSetting){
        parent::__construct($request, $response,self::ENDPOINT,$defaultEshopSetting);
    }

    /**
     * @return CustomersRequest
     */
    public function getRequest(){
        return $this->isCorrectRequest(parent::getRequest());
    }

    /**
     * @return CustomersResponse
     */
    public function getResponse(){
        return $this->isCorrectResponse(parent::getResponse());
    }

    /**
     * @param IRequest $request
     * @return CustomersRequest
     * @throws WrongRequestObjectException
     */
    private function isCorrectRequest(IRequest $request) {
        if ($request instanceof CustomersRequest) return $request;
        throw new WrongRequestObjectException(get_class($request),get_class(new CustomersRequest()));
    }

    /**
     * @param IResponse $response
     * @return CustomersResponse
     * @throws WrongResponseObjectException
     */
    private function isCorrectResponse(IResponse $response) {
        if ($response instanceof CustomersResponse) return $response;
        throw new WrongResponseObjectException(get_class($response),get_class(new CustomersResponse()));
    }


}