<?php

namespace Monkey\ImportEshopApi\Endpoint;

use Monkey\ImportEshopApi\Base\ADataMiner;
use Monkey\ImportEshopApi\Base\ADefaultEshopSetting;
use Monkey\ImportEshopApi\Exceptions\WrongRequestObjectException;
use Monkey\ImportEshopApi\Exceptions\WrongResponseObjectException;
use Monkey\ImportEshopDataObjects\Interfaces\IRequest;
use Monkey\ImportEshopDataObjects\Interfaces\IResponse;
use Monkey\ImportEshopDataObjects\Request\OrderStatusesRequest;
use Monkey\ImportEshopDataObjects\Response\OrderStatusesResponse;

/**
 * Class AOrderStatuses
 * @package Monkey\ImportEshopApi\Endpoint
 */
abstract class AOrderStatuses extends ADataMiner
{

    const ENDPOINT = "orderstatuses";

    /**
     * AOrders constructor.
     * @param IRequest $request
     * @param IResponse $response
     * @param ADefaultEshopSetting $defaultEshopSetting
     */
    public function __construct($request,$response, $defaultEshopSetting){
        parent::__construct($request, $response,self::ENDPOINT,$defaultEshopSetting);
    }

    /**
     * @return OrderStatusesRequest
     */
    public function getRequest(){
        return $this->isCorrectRequest(parent::getRequest());
    }

    /**
     * @return OrderStatusesResponse
     */
    public function getResponse(){
        return $this->isCorrectResponse(parent::getResponse());
    }

    /**
     * @param IRequest $request
     * @return OrderStatusesRequest
     * @throws WrongRequestObjectException
     */
    private function isCorrectRequest(IRequest $request) {
        if ($request instanceof OrderStatusesRequest) return $request;
        throw new WrongRequestObjectException(get_class($request),get_class(new OrderStatusesRequest()));
    }

    /**
     * @param IResponse $response
     * @return OrderStatusesResponse
     * @throws WrongResponseObjectException
     */
    private function isCorrectResponse(IResponse $response) {
        if ($response instanceof OrderStatusesResponse) return $response;
        throw new WrongResponseObjectException(get_class($response),get_class(new OrderStatusesResponse()));
    }


}