<?php

namespace Monkey\ImportEshopApi\Endpoint;

use Monkey\ImportEshopApi\Base\ADataMiner;
use Monkey\ImportEshopApi\Base\ADefaultEshopSetting;
use Monkey\ImportEshopApi\Exceptions\WrongRequestObjectException;
use Monkey\ImportEshopApi\Exceptions\WrongResponseObjectException;
use Monkey\ImportEshopDataObjects\Interfaces\IRequest;
use Monkey\ImportEshopDataObjects\Interfaces\IResponse;
use Monkey\ImportEshopDataObjects\Request\OrdersRequest;
use Monkey\ImportEshopDataObjects\Response\OrdersResponse;

/**
 * Class AOrders
 * @package Monkey\ImportEshopApi\Endpoint
 */
abstract class AOrders extends ADataMiner
{

    const ENDPOINT = "orders";
    const ESHOP_SOURCE = "eshop";

    /**
     * AOrders constructor.
     * @param IRequest $request
     * @param IResponse $response
     */

    /**
     * AOrders constructor.
     * @param IRequest $request
     * @param IResponse $response
     * @param ADefaultEshopSetting $defaultEshopSetting
     */
    public function __construct($request,$response, $defaultEshopSetting){
        parent::__construct($request, $response,self::ENDPOINT,$defaultEshopSetting);
    }

    /**
     * @return OrdersRequest
     */
    public function getRequest(){
        return $this->isCorrectRequest(parent::getRequest());
    }

    /**
     * @return OrdersResponse
     */
    public function getResponse(){
        return $this->isCorrectResponse(parent::getResponse());
    }

    /**
     * @param IRequest $request
     * @return OrdersRequest
     * @throws WrongRequestObjectException
     */
    private function isCorrectRequest(IRequest $request) {
        if ($request instanceof OrdersRequest) return $request;
        throw new WrongRequestObjectException(get_class($request),get_class(new OrdersRequest()));
    }

    /**
     * @param IResponse $response
     * @return OrdersResponse
     * @throws WrongResponseObjectException
     */
    private function isCorrectResponse(IResponse $response) {
        if ($response instanceof OrdersResponse) return $response;
        throw new WrongResponseObjectException(get_class($response),get_class(new OrdersResponse()));
    }


}