<?php

namespace Monkey\ImportEshopApi\Endpoint;

use Monkey\ImportEshopApi\Base\ADataMiner;
use Monkey\ImportEshopApi\Base\ADefaultEshopSetting;
use Monkey\ImportEshopApi\Exceptions\WrongRequestObjectException;
use Monkey\ImportEshopApi\Exceptions\WrongResponseObjectException;
use Monkey\ImportEshopDataObjects\Interfaces\IRequest;
use Monkey\ImportEshopDataObjects\Interfaces\IResponse;
use Monkey\ImportEshopDataObjects\Request\ReviewsRequest;
use Monkey\ImportEshopDataObjects\Response\ReviewsResponse;

/**
 * Class AReviews
 * @package Monkey\ImportEshopApi\Endpoint
 */
abstract class AReviews extends ADataMiner
{

    const ENDPOINT = "reviews";

    /**
     * AOrders constructor.
     * @param IRequest $request
     * @param IResponse $response
     * @param ADefaultEshopSetting $defaultEshopSetting
     */
    public function __construct($request,$response, $defaultEshopSetting){
        parent::__construct($request, $response,self::ENDPOINT,$defaultEshopSetting);
    }

    /**
     * @return ReviewsRequest
     */
    public function getRequest(){
        return $this->isCorrectRequest(parent::getRequest());
    }

    /**
     * @return ReviewsResponse
     */
    public function getResponse(){
        return $this->isCorrectResponse(parent::getResponse());
    }

    /**
     * @param IRequest $request
     * @return ReviewsRequest
     * @throws WrongRequestObjectException
     */
    private function isCorrectRequest(IRequest $request) {
        if ($request instanceof ReviewsRequest) return $request;
        throw new WrongRequestObjectException(get_class($request),get_class(new ReviewsRequest()));
    }

    /**
     * @param IResponse $response
     * @return ReviewsResponse
     * @throws WrongResponseObjectException
     */
    private function isCorrectResponse(IResponse $response) {
        if ($response instanceof ReviewsResponse) return $response;
        throw new WrongResponseObjectException(get_class($response),get_class(new ReviewsResponse()));
    }


}