<?php

namespace Monkey\ImportEshopApi\Endpoint;

use Monkey\ImportEshopApi\Base\ADataMiner;
use Monkey\ImportEshopApi\Base\ADefaultEshopSetting;
use Monkey\ImportEshopApi\Exceptions\WrongRequestObjectException;
use Monkey\ImportEshopApi\Exceptions\WrongResponseObjectException;
use Monkey\ImportEshopDataObjects\Interfaces\IRequest;
use Monkey\ImportEshopDataObjects\Interfaces\IResponse;
use Monkey\ImportEshopDataObjects\Request\ShipmentsRequest;
use Monkey\ImportEshopDataObjects\Response\ShipmentsResponse;

/**
 * Class AShipments
 * @package Monkey\ImportEshopApi\Endpoint
 */
abstract class AShipments extends ADataMiner
{

    const ENDPOINT = "shipments";

    /**
     * AOrders constructor.
     * @param IRequest $request
     * @param IResponse $response
     * @param ADefaultEshopSetting $defaultEshopSetting
     */
    public function __construct($request,$response, $defaultEshopSetting){
        parent::__construct($request, $response,self::ENDPOINT,$defaultEshopSetting);
    }

    /**
     * @return ShipmentsRequest
     */
    public function getRequest(){
        return $this->isCorrectRequest(parent::getRequest());
    }

    /**
     * @return ShipmentsResponse
     */
    public function getResponse(){
        return $this->isCorrectResponse(parent::getResponse());
    }

    /**
     * @param IRequest $request
     * @return ShipmentsRequest
     * @throws WrongRequestObjectException
     */
    private function isCorrectRequest(IRequest $request) {
        if ($request instanceof ShipmentsRequest) return $request;
        throw new WrongRequestObjectException(get_class($request),get_class(new ShipmentsRequest()));
    }

    /**
     * @param IResponse $response
     * @return ShipmentsResponse
     * @throws WrongResponseObjectException
     */
    private function isCorrectResponse(IResponse $response) {
        if ($response instanceof ShipmentsResponse) return $response;
        throw new WrongResponseObjectException(get_class($response),get_class(new ShipmentsResponse()));
    }


}