<?php

namespace Monkey\ImportEshopApi\Request;

use Monkey\ImportEshopApi\Exceptions\WrongRequestObjectException;
use Monkey\ImportEshopDataObjects\Base\Request;
use NilPortugues\Serializer\Serializer;

/**
 * Class RequestBuilder
 * @package Monkey\ImportEshopApi\Request
 */
class RequestBuilder {

    /**
     * @var Serializer
     */
    private $serializer;

    /**
     * @var Request
     */
    private $request;

    public function __construct($serializer){
        $this->setSerializer($serializer);
    }

    /**
     * @param string $requestString
     * @return Request
     * @throws WrongRequestObjectException
     */
    public function build($requestString = ""){
        $this->requestFactory($requestString);
        return $this->getRequest();
    }

    /**
     * @param $requestString
     * @throws WrongRequestObjectException
     */
    private function requestFactory($requestString){
        $request = $this->getSerializer()->unserialize($requestString);
        if(!($request instanceof Request)){
            throw new WrongRequestObjectException(get_class($request),"Request");
        }
        $this->setRequest($request);
    }

    /**
     * @return Serializer
     */
    public function getSerializer(){
        return $this->serializer;
    }

    /**
     * @param Serializer $serializer
     * @return RequestBuilder
     */
    private function setSerializer($serializer){
        $this->serializer = $serializer;
        return $this;
    }

    /**
     * @return Request
     */
    public function getRequest(){
        return $this->request;
    }

    /**
     * @param Request $request
     * @return RequestBuilder
     */
    private function setRequest($request){
        $this->request = $request;
        return $this;
    }



}