<?php

namespace Monkey\ImportEshopApi\Response;

use Monkey\ImportEshopDataObjects\Base\Response;
use Monkey\ImportEshopDataObjects\Interfaces\IRequest;
use Monkey\ImportEshopDataObjects\Response\Orders;
use NilPortugues\Serializer\JsonSerializer;
use NilPortugues\Serializer\Serializer;

/**
 * Class ResponseBuilder
 * @package Monkey\ImportEshopApi\Response
 */
class ResponseBuilder {

    /**
     * @var Serializer
     */
    private $serializer;

    /**
     * @var Response
     */
    private $response;

    /**
     * ResponseBuilder constructor.
     * @param Serializer $serializer
     */
    public function __construct($serializer){
        $this->setSerializer($serializer);
    }

    public function render(){
        $this->http_response_code($this->getResponse()->getStatusCode());

        if(substr(php_sapi_name(), 0, 3) == 'cli'){
            vd($this->getResponse());
            exit;
        }

        //other servers, include http
        if($this->getSerializer() instanceof JsonSerializer){
            header('Content-Type: application/json');
            echo $this->getSerializer()->serialize($this->getResponse());
        }else{
            vd($this->getResponse());
        }
        exit;
    }

    /**
     * @param array $messages
     */
    public function addErrors($messages = array()){
        foreach($messages as $message){
            $this->getResponse()->addErrorMessage($message);
        }

    }

    /**
     * @return Serializer
     */
    public function getSerializer(){
        return $this->serializer;
    }

    /**
     * @param Serializer $serializer
     * @return ResponseBuilder
     */
    private function setSerializer($serializer){
        $this->serializer = $serializer;
        return $this;
    }

    /**
     * @param IRequest $request
     */
    public function setupResponse(IRequest $request){

        $responseName = str_replace("Request", "Response", get_class($request));
        $response = new $responseName();
        $this->setResponse($response);
    }

    /**
     * @return Response
     */
    public function getResponse(){
        return $this->response;
    }

    /**
     * @param Response $response
     * @return ResponseBuilder
     */
    public function setResponse($response){
        $this->response = $response;
        return $this;
    }


    private function http_response_code($code = NULL){

        if (is_null($code)) {
            $code = (isset($GLOBALS['http_response_code']) ? $GLOBALS['http_response_code'] : 200);
        }

        if (!function_exists('http_response_code')) {
            switch ($code) {
                case 100:
                    $text = 'Continue';
                    break;
                case 101:
                    $text = 'Switching Protocols';
                    break;
                case 200:
                    $text = 'OK';
                    break;
                case 201:
                    $text = 'Created';
                    break;
                case 202:
                    $text = 'Accepted';
                    break;
                case 203:
                    $text = 'Non-Authoritative Information';
                    break;
                case 204:
                    $text = 'No Content';
                    break;
                case 205:
                    $text = 'Reset Content';
                    break;
                case 206:
                    $text = 'Partial Content';
                    break;
                case 300:
                    $text = 'Multiple Choices';
                    break;
                case 301:
                    $text = 'Moved Permanently';
                    break;
                case 302:
                    $text = 'Moved Temporarily';
                    break;
                case 303:
                    $text = 'See Other';
                    break;
                case 304:
                    $text = 'Not Modified';
                    break;
                case 305:
                    $text = 'Use Proxy';
                    break;
                case 400:
                    $text = 'Bad Request';
                    break;
                case 401:
                    $text = 'Unauthorized';
                    break;
                case 402:
                    $text = 'Payment Required';
                    break;
                case 403:
                    $text = 'Forbidden';
                    break;
                case 404:
                    $text = 'Not Found';
                    break;
                case 405:
                    $text = 'Method Not Allowed';
                    break;
                case 406:
                    $text = 'Not Acceptable';
                    break;
                case 407:
                    $text = 'Proxy Authentication Required';
                    break;
                case 408:
                    $text = 'Request Time-out';
                    break;
                case 409:
                    $text = 'Conflict';
                    break;
                case 410:
                    $text = 'Gone';
                    break;
                case 411:
                    $text = 'Length Required';
                    break;
                case 412:
                    $text = 'Precondition Failed';
                    break;
                case 413:
                    $text = 'Request Entity Too Large';
                    break;
                case 414:
                    $text = 'Request-URI Too Large';
                    break;
                case 415:
                    $text = 'Unsupported Media Type';
                    break;
                case 500:
                    $text = 'Internal Server Error';
                    break;
                case 501:
                    $text = 'Not Implemented';
                    break;
                case 502:
                    $text = 'Bad Gateway';
                    break;
                case 503:
                    $text = 'Service Unavailable';
                    break;
                case 504:
                    $text = 'Gateway Time-out';
                    break;
                case 505:
                    $text = 'HTTP Version not supported';
                    break;
                default:
                    exit('Unknown http status code "' . htmlentities($code) . '"');
                    break;
            }

            $protocol = (isset($_SERVER['SERVER_PROTOCOL']) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.1');

            header($protocol . ' ' . $code . ' ' . $text);

            $GLOBALS['http_response_code'] = $code;

        } else {
            http_response_code($code);
        }
        return $code;

    }


}