<?php

namespace Monkey\ImportEshopDataObjects\Base;


use Monkey\ImportEshopDataObjects\Entity\Simple\Alias;
use Monkey\ImportEshopDataObjects\Entity\Simple\Condition;
use Monkey\ImportEshopDataObjects\Entity\Simple\Join;
use Monkey\ImportEshopDataObjects\Interfaces\IDifferences;

/**
 * Class Differences
 * @package Monkey\ImportEshopDataObjects\Base
 */
class Differences implements IDifferences {
    const TYPE_COLUMNS = 1;
    const TYPE_TABLE_ALIAS = 2;
    const TYPE_CONDITION = 3;

    /**
     * @var Join[] $tableAliases
     */
    protected $tableAliases = array();

    /**
     * @var Alias[] $columns
     */
    protected $columns = array();

    /**
     * @var Condition[] $additionalConditions
     */
    protected $additionalConditions = array();

    /**
     * @return Join[]
     */
    public function getTableAliases() {
        return $this->tableAliases;
    }

    /**
     * @param Join $alias
     * @return Differences
     */
    public function addTableAlias($alias) {
        $this->tableAliases[] = $alias;
        return $this;
    }

    /**
     * @return Alias[]
     */
    public function getColumns() {
        return $this->columns;
    }

    /**
     * @param Alias $column
     * @return Differences
     */
    public function addColumn($column) {
        $this->columns[] = $column;
        return $this;
    }

    /**
     * @return Condition[]
     */
    public function getAdditionalConditions() {
        return $this->additionalConditions;
    }

    /**
     * @param Condition $condition
     * @return Differences
     */
    public function addAdditionalCondition($condition) {
        $this->additionalConditions[] = $condition;
        return $this;
    }
}