<?php

namespace Monkey\ImportEshopDataObjects\Base;


use Monkey\ImportEshopDataObjects\Entity\Simple\CurrentSetting;
use Monkey\ImportEshopDataObjects\Interfaces\IRequest;

/**
 * Class Request
 * @package Monkey\ImportEshopDataObjects\Base
 */
abstract class Request implements IRequest {
    const DEFAULT_LANGUAGE = "en";

    /**
     * @var int $page
     */
    private $page = 1;

    /**
     * @var int $limit
     */
    private $limit = 100;

    /**
     * @var Differences $differences
     */
    private $differences;

    /**
     * @var string $hash
     */
    private $hash;

    /**
     * @var bool $debug
     */
    private $debug = false;

    /**
     * @var int $sinceId
     */
    private $sinceId;

    /**
     * @var int $maxId
     */
    private $maxId;

    /**
     * @var string $createdFrom
     */
    private $createdFrom;

    /**
     * @var string $createdTo
     */
    private $createdTo;

    /**
     * @var string $updatedFrom
     */
    private $updatedFrom;

    /**
     * @var string $updatedTo
     */
    private $updatedTo;

    /**
     * @var string $id
     */
    private $id;

    /**
     * @var string $defaultLanguageIso
     */
    private $defaultLanguageIso;

    /**
     * @var CurrentSetting[] $currentDefaultSettings
     */
    private $currentDefaultSettings = array();

    /**
     * @return int
     */
    public function getPage() {
        return $this->page;
    }

    /**
     * @param int $page
     * @return static
     */
    public function setPage($page) {
        $this->page = $page;
        return $this;
    }

    /**
     * Increments page by 1
     * @return static
     */
    public function nextPage() {
        $this->page++;
        return $this;
    }

    /**
     * @return int
     */
    public function getLimit() {
        return $this->limit;
    }

    /**
     * @param int $limit
     * @return static
     */
    public function setLimit($limit) {
        $this->limit = $limit;
        return $this;
    }

    /**
     * @return Differences
     */
    public function getDifferences() {
        return $this->differences;
    }

    /**
     * @param Differences $differences
     * @return static
     */
    public function setDifferences(Differences $differences) {
        $this->differences = $differences;
        return $this;
    }

    /**
     * @return string
     */
    public function getHash() {
        return $this->hash;
    }

    /**
     * @param string $hash
     * @return static
     */
    public function setHash($hash) {
        $this->hash = $hash;
        return $this;
    }

    /**
     * @return bool
     */
    public function isDebug() {
        return $this->debug;
    }

    /**
     * @param bool $debug
     * @return static
     */
    public function setDebug($debug) {
        $this->debug = $debug;
        return $this;
    }

    /**
     * @return int
     */
    protected function getSinceId() {
        return $this->sinceId;
    }

    /**
     * @param int $sinceId
     * @return static
     */
    protected function setSinceId($sinceId) {
        $this->sinceId = $sinceId;
        return $this;
    }

    /**
     * @return int
     */
    protected function getMaxId() {
        return $this->maxId;
    }

    /**
     * @param int $maxId
     * @return static
     */
    protected function setMaxId($maxId) {
        $this->maxId = $maxId;
        return $this;
    }

    /**
     * @return string
     */
    protected function getCreatedFrom() {
        return $this->createdFrom;
    }

    /**
     * @param string $createdFrom
     * @return static
     */
    protected function setCreatedFrom($createdFrom) {
        $this->createdFrom = $createdFrom;
        return $this;
    }

    /**
     * @return string
     */
    protected function getCreatedTo() {
        return $this->createdTo;
    }

    /**
     * @param string $createdTo
     * @return static
     */
    protected function setCreatedTo($createdTo) {
        $this->createdTo = $createdTo;
        return $this;
    }

    /**
     * @return string
     */
    protected function getUpdatedFrom() {
        return $this->updatedFrom;
    }

    /**
     * @param string $updatedFrom
     * @return static
     */
    protected function setUpdatedFrom($updatedFrom) {
        $this->updatedFrom = $updatedFrom;
        return $this;
    }

    /**
     * @return string
     */
    protected function getUpdatedTo() {
        return $this->updatedTo;
    }

    /**
     * @param string $updatedTo
     * @return static
     */
    protected function setUpdatedTo($updatedTo) {
        $this->updatedTo = $updatedTo;
        return $this;
    }

    /**
     * @return string
     */
    public function getId() {
        return $this->id;
    }

    /**
     * @param string $id
     * @return static
     */
    public function setId($id) {
        $this->id = $id;
        return $this;
    }

    /**
     * @param string $conditionsString
     * @return string
     */
    public function setupVariables($conditionsString) {
        foreach ($this->getAllowedConditionVariables() as $variable) {
            $getName = $this->prepareGetFunctionName($variable);

            if (method_exists($this, $getName)) {
                $value = $this->$getName();
                $conditionsString = str_replace($variable, $value, $conditionsString);
            }

            $keyName = $this->prepareKeyCurrentSettingName($variable);

            if ($this->hasCurrentDefaultSetting($keyName)) {
                $value = $this->getCurrentDefaultSetting($keyName);
                $conditionsString = str_replace($variable, $value->getValue(), $conditionsString);
            }
        }
        return $conditionsString;
    }

    /**
     * @return array
     */
    private function getAllowedConditionVariables() {
        $default = array(
            "[[createdFrom]]",
            "[[createdTo]]",
            "[[updatedFrom]]",
            "[[updatedTo]]",
            "[[sinceId]]",
            "[[maxId]]",
            "[[id]]",
            "[[foreignId]]",
        );

        foreach (array_keys($this->getCurrentDefaultSettings()) as $keyName) {
            $default[] = "[[{$keyName}]]";
        }

        return $default;
    }

    /**
     * @param string $variableName
     * @return string
     */
    private function prepareGetFunctionName($variableName) {
        $variableName = str_replace("[[", "", $variableName);
        $variableName = str_replace("]]", "", $variableName);
        return "get" . ucfirst($variableName);
    }

    /**
     * @param string $variableName
     * @return string
     */
    private function prepareKeyCurrentSettingName($variableName) {
        $variableName = str_replace("[[", "", $variableName);
        $variableName = str_replace("]]", "", $variableName);
        return $variableName;
    }

    /**
     * @return string
     */
    public function getDefaultLanguageIso() {
        if(is_null($this->defaultLanguageIso)) {
            return self::DEFAULT_LANGUAGE;
        }
        return $this->defaultLanguageIso;
    }

    /**
     * @param string $defaultLanguageIso
     * @return static
     */
    public function setDefaultLanguageIso($defaultLanguageIso) {
        $this->defaultLanguageIso = $defaultLanguageIso;
        return $this;
    }

    /**
     * @return CurrentSetting[]
     */
    public function getCurrentDefaultSettings() {
        return $this->currentDefaultSettings;
    }

    /**
     * @param CurrentSetting[] $currentDefaultSettings
     * @return CurrentSetting[]
     */
    public function setCurrentDefaultSettings($currentDefaultSettings = array()) {
        return $this->currentDefaultSettings = $currentDefaultSettings;
    }

    /**
     * @param string $settingKey
     * @return CurrentSetting|null
     */
    public function getCurrentDefaultSetting($settingKey) {
        if (array_key_exists($settingKey, $this->currentDefaultSettings)) {
            return $this->currentDefaultSettings[$settingKey];
        }

        return null;
    }

    /**
     * @param string $settingKey
     * @return bool
     */
    public function hasCurrentDefaultSetting($settingKey) {
        return array_key_exists($settingKey, $this->currentDefaultSettings);
    }

    /**
     * @param CurrentSetting $currentDefaultSettings
     * @return static
     */
    public function addCurrentDefaultSettings(CurrentSetting $currentDefaultSettings) {
        $this->currentDefaultSettings[$currentDefaultSettings->getKey()] = $currentDefaultSettings;
        return $this;
    }
}