<?php

namespace Monkey\ImportEshopDataObjects\Base;


use Monkey\ImportEshopDataObjects\Interfaces\IResponse;

/**
 * Class Response
 * @package Monkey\ImportEshopDataObjects\Base
 */
abstract class Response implements IResponse {
    /**
     * @var int $statusCode
     */
    private $statusCode = 200;

    /**
     * @var string $message
     */
    private $message = "OK";

    /**
     * @var array $content
     */
    private $content = array();

    /**
     * @var string $schema
     */
    private $schema;

    /**
     * @var string $query
     */
    private $query;

    /**
     * @var string $phpVersion
     */
    private $phpVersion;

    /**
     * @var string $eshopPlatform
     */
    private $eshopPlatform;

    /**
     * @var string $eshopPlatformVersion
     */
    private $eshopPlatformVersion;

    /**
     * @var string $eshopPluginVersion
     */
    private $eshopPluginVersion;

    /**
     * @var string $importEshopApiVersion
     */
    private $importEshopApiVersion;

    /**
     * @var string $importEshopDataObjectsVersion
     */
    private $importEshopDataObjectsVersion;

    /**
     * @var array $errorMessages
     */
    private $errorMessages = array();

    /**
     * @return int
     */
    public function getStatusCode() {
        return $this->statusCode;
    }

    /**
     * @param int $statusCode
     * @return static
     */
    public function setStatusCode($statusCode) {
        $this->statusCode = $statusCode;
        return $this;
    }

    /**
     * @return string
     */
    public function getMessage() {
        return $this->message;
    }

    /**
     * @param string $message
     * @return static
     */
    public function setMessage($message) {
        $this->message = $message;
        return $this;
    }

    /**
     * @return array
     */
    public function getContent() {
        return $this->content;
    }

    /**
     * @param array $content
     * @return static
     */
    public function setContent(array $content) {
        $this->content = $content;
        return $this;
    }

    /**
     * @return Entity
     */
    public function addNewEntity() {
        $entity = $this->getEntity();
        $this->content[] = $entity;
        return $entity;
    }

    /**
     * @return Entity
     */
    abstract protected function getEntity();

    /**
     * @return string
     */
    public function getSchema() {
        return $this->schema;
    }

    /**
     * @param string $schema
     * @return static
     */
    public function setSchema($schema) {
        $this->schema = $schema;
        return $this;
    }

    /**
     * @return string
     */
    public function getQuery() {
        return $this->query;
    }

    /**
     * @param string $query
     * @return static
     */
    public function setQuery($query) {
        $this->query = $query;
        return $this;
    }

    /**
     * @return string
     */
    public function getPhpVersion() {
        return $this->phpVersion;
    }

    /**
     * @param string $phpVersion
     * @return static
     */
    public function setPhpVersion($phpVersion) {
        $this->phpVersion = $phpVersion;
        return $this;
    }

    /**
     * @return string
     */
    public function getEshopPlatform() {
        return $this->eshopPlatform;
    }

    /**
     * @param string $eshopPlatform
     * @return static
     */
    public function setEshopPlatform($eshopPlatform) {
        $this->eshopPlatform = $eshopPlatform;
        return $this;
    }

    /**
     * @return string
     */
    public function getEshopPlatformVersion() {
        return $this->eshopPlatformVersion;
    }

    /**
     * @param string $eshopPlatformVersion
     * @return static
     */
    public function setEshopPlatformVersion($eshopPlatformVersion) {
        $this->eshopPlatformVersion = $eshopPlatformVersion;
        return $this;
    }

    /**
     * @return string
     */
    public function getEshopPluginVersion() {
        return $this->eshopPluginVersion;
    }

    /**
     * @param string $eshopPluginVersion
     * @return static
     */
    public function setEshopPluginVersion($eshopPluginVersion) {
        $this->eshopPluginVersion = $eshopPluginVersion;
        return $this;
    }

    /**
     * @return string
     */
    public function getImportEshopApiVersion() {
        return $this->importEshopApiVersion;
    }

    /**
     * @param string $importEshopApiVersion
     * @return static
     */
    public function setImportEshopApiVersion($importEshopApiVersion) {
        $this->importEshopApiVersion = $importEshopApiVersion;
        return $this;
    }

    /**
     * @return string
     */
    public function getImportEshopDataObjectsVersion() {
        return $this->importEshopDataObjectsVersion;
    }

    /**
     * @param string $importEshopDataObjectsVersion
     * @return static
     */
    public function setImportEshopDataObjectsVersion($importEshopDataObjectsVersion) {
        $this->importEshopDataObjectsVersion = $importEshopDataObjectsVersion;
        return $this;
    }

    /**
     * @return array
     */
    public function getErrorMessages() {
        return $this->errorMessages;
    }

    /**
     * @param array $errorMessages
     * @return static
     */
    public function setErrorMessages($errorMessages) {
        $this->errorMessages = $errorMessages;
        return $this;
    }

    /**
     * @param string $message
     * @return static
     */
    public function addErrorMessage($message) {
        $this->errorMessages[] = $message;
        return $this;
    }
}