<?php

namespace Monkey\ImportEshopDataObjects\Entity;


use Monkey\ImportEshopDataObjects\Base\Entity;
use Monkey\ImportEshopDataObjects\Entity\Simple\Price;

/**
 * Class Coupon
 * @package Monkey\ImportEshopDataObjects\Entity
 */
class Coupon extends Entity {
    const TYPE_AMOUNT = 'amount';
    const TYPE_PERCENTAGE = 'percentage';
    const SHIPMENT_NONE = 'none';
    const SHIPMENT_DISCOUNT = 'discount';
    const SHIPMENT_FREE = 'free';

    /**
     * @var string $code
     */
    private $code;

    /**
     * @var string $type
     */
    private $type;

    /**
     * @var float $amount
     */
    private $amount;

    /**
     * @var int $usageCountLimit
     */
    private $usageCountLimit = 1;
    /**
     * @var Price $orderMinimumAmount
     */
    private $orderMinimumAmount;

    /**
     * @var string $shipment
     */
    private $shipment;

    /**
     * @var string $startDate
     */
    private $startDate;

    /**
     * @var string $endDate
     */
    private $endDate;

    /**
     * @var array $products
     */
    private $products = array();

    /**
     * @var array $categories
     */
    private $categories = array();

    /**
     * @return string
     */
    public function getCode() {
        return $this->code;
    }

    /**
     * @param string $code
     * @return Coupon
     */
    public function setCode($code) {
        $this->code = $code;
        return $this;
    }

    /**
     * @return string
     */
    public function getType() {
        return $this->type;
    }

    /**
     * @return Coupon
     */
    public function setTypeToAmount() {
        $this->type = self::TYPE_AMOUNT;
        return $this;
    }

    /**
     * @return Coupon
     */
    public function setTypeToPercentage() {
        $this->type = self::TYPE_PERCENTAGE;
        return $this;
    }

    /**
     * @return float
     */
    public function getAmount() {
        return $this->amount;
    }

    /**
     * @param float $amount
     * @return Coupon
     */
    public function setAmount($amount) {
        $this->amount = $amount;
        return $this;
    }

    /**
     * @return int
     */
    public function getUsageCountLimit() {
        return $this->usageCountLimit;
    }

    /**
     * @param int $usageCountLimit
     * @return Coupon
     */
    public function setUsageCountLimit($usageCountLimit) {
        $this->usageCountLimit = $usageCountLimit;
        return $this;
    }

    /**
     * @return Price
     */
    public function getOrderMinimumAmount() {
        return $this->orderMinimumAmount;
    }

    /**
     * @param Price $orderMinimumAmount
     * @return Coupon
     */
    public function setOrderMinimumAmount(Price $orderMinimumAmount) {
        $this->orderMinimumAmount = $orderMinimumAmount;
        return $this;
    }

    /**
     * @return string
     */
    public function getShipment() {
        return $this->shipment;
    }

    /**
     * @return Coupon
     */
    public function setShipmentNone() {
        $this->shipment = self::SHIPMENT_NONE;
        return $this;
    }

    /**
     * @return Coupon
     */
    public function setShipmentDiscount() {
        $this->shipment = self::SHIPMENT_DISCOUNT;
        return $this;
    }

    /**
     * @return Coupon
     */
    public function setShipmentFree() {
        $this->shipment = self::SHIPMENT_FREE;
        return $this;
    }

    /**
     * @return string
     */
    public function getStartDate() {
        return $this->startDate;
    }

    /**
     * @param string $startDate
     * @return Coupon
     */
    public function setStartDate($startDate) {
        $this->startDate = $startDate;
        return $this;
    }

    /**
     * @return string
     */
    public function getEndDate() {
        return $this->endDate;
    }

    /**
     * @param string $endDate
     * @return Coupon
     */
    public function setEndDate($endDate) {
        $this->endDate = $endDate;
        return $this;
    }

    /**
     * @return array
     */
    public function getProducts() {
        return $this->products;
    }

    /**
     * @param string $product
     * @return Coupon
     */
    public function addProduct($product) {
        $this->products[] = $product;
        return $this;
    }

    /**
     * @return array
     */
    public function getCategories() {
        return $this->categories;
    }

    /**
     * @param string $category
     * @return Coupon
     */
    public function addCategory($category) {
        $this->categories[] = $category;
        return $this;
    }
}