<?php

namespace Monkey\ImportEshopDataObjects\Entity;


use Monkey\ImportEshopDataObjects\Base\Entity;
use Monkey\ImportEshopDataObjects\Entity\Simple\Discount;
use Monkey\ImportEshopDataObjects\Entity\Simple\Payment;
use Monkey\ImportEshopDataObjects\Entity\Simple\Price;
use Monkey\ImportEshopDataObjects\Entity\Simple\Shipment;

/**
 * Class Order
 * @package Monkey\ImportEshopDataObjects\Entity
 */
class Order extends Entity {
    /**
     * @var string $visibleOrderId
     */
    private $visibleOrderId;

    /**
     * @var string $sourceType
     */
    private $sourceType;

    /**
     * @var string $originalOrderId
     */
    private $originalOrderId;

    /**
     * @var string $orderStatusId
     */
    private $orderStatusId;

    /**
     * @var string $orderStatusId
     */
    private $orderStatusName;

    /**
     * @var Price $price
     */
    private $price;

    /**
     * @var Payment $payment
     */
    private $payment;

    /**
     * @var Shipment $shipment
     */
    private $shipment;

    /**
     * @var string $note
     */
    private $note;

    /**
     * @var Customer $customer
     */
    private $customer;

    /**
     * @var Discount[] $discounts
     */
    private $discounts = array();

    /**
     * @return string
     */
    public function getVisibleOrderId() {
        return $this->visibleOrderId;
    }

    /**
     * @param string $visibleOrderId
     * @return Order
     */
    public function setVisibleOrderId($visibleOrderId) {
        $this->visibleOrderId = $visibleOrderId;
        return $this;
    }

    /**
     * @return string
     */
    public function getSourceType() {
        return $this->sourceType;
    }

    /**
     * @param string $sourceType
     * @return Order
     */
    public function setSourceType($sourceType) {
        $this->sourceType = $sourceType;
        return $this;
    }

    /**
     * @return string
     */
    public function getOriginalOrderId() {
        return $this->originalOrderId;
    }

    /**
     * @param string $originalOrderId
     * @return Order
     */
    public function setOriginalOrderId($originalOrderId) {
        $this->originalOrderId = $originalOrderId;
        return $this;
    }

    /**
     * @return string
     */
    public function getOrderStatusId() {
        return $this->orderStatusId;
    }

    /**
     * @param string $orderStatusId
     * @return Order
     */
    public function setOrderStatusId($orderStatusId) {
        $this->orderStatusId = $orderStatusId;
        return $this;
    }

    /**
     * @return string
     */
    public function getOrderStatusName(){
        return $this->orderStatusName;
    }

    /**
     * @param string $orderStatusName
     * @return Order
     */
    public function setOrderStatusName($orderStatusName){
        $this->orderStatusName = $orderStatusName;
        return $this;
    }

    /**
     * @return Price
     */
    public function getPrice() {
        if (is_null($this->price)) {
            $this->setPrice(new Price());
        }

        return $this->price;
    }

    /**
     * @param Price $price
     * @return Order
     */
    public function setPrice(Price $price) {
        $this->price = $price;
        return $this;
    }

    /**
     * @return Payment
     */
    public function getPayment() {
        if (is_null($this->payment)) {
            $this->setPayment(new Payment());
        }

        return $this->payment;
    }

    /**
     * @param Payment $payment
     * @return Order
     */
    public function setPayment(Payment $payment) {
        $this->payment = $payment;
        return $this;
    }

    /**
     * @return Shipment
     */
    public function getShipment() {
        if (is_null($this->shipment)) {
            $this->setShipment(new Shipment());
        }

        return $this->shipment;
    }

    /**
     * @param Shipment $shipment
     * @return Order
     */
    public function setShipment(Shipment $shipment) {
        $this->shipment = $shipment;
        return $this;
    }

    /**
     * @return string
     */
    public function getNote() {
        return $this->note;
    }

    /**
     * @param string $note
     * @return Order
     */
    public function setNote($note) {
        $this->note = $note;
        return $this;
    }

    /**
     * @return Customer
     */
    public function getCustomer() {
        if (is_null($this->customer)) {
            $this->setCustomer(new Customer());
        }

        return $this->customer;
    }

    /**
     * @param Customer $customer
     * @return Order
     */
    public function setCustomer(Customer $customer) {
        $this->customer = $customer;
        return $this;
    }

    /**
     * @return Discount[]
     */
    public function getDiscounts() {
        return $this->discounts;
    }

    /**
     * @param Discount $discount
     * @return Order
     */
    public function addDiscount(Discount $discount) {
        $this->discounts[] = $discount;
        return $this;
    }
}