<?php

namespace Monkey\ImportEshopDataObjects\Entity;


use Monkey\ImportEshopDataObjects\Base\Entity;
use Monkey\ImportEshopDataObjects\Entity\Simple\Discount;
use Monkey\ImportEshopDataObjects\Entity\Simple\Price;

/**
 * Class OrderItem
 * @package Monkey\ImportEshopDataObjects\Entity
 */
class OrderItem extends Entity {
    /**
     * @var string $orderId
     */
    private $orderId;

    /**
     * @var string $productId
     */
    private $productId;

    /**
     * @var string $variantId
     */
    private $variantId;

    /**
     * @var Price $price
     */
    private $price;

    /**
     * @var Price $purchasePrice
     */
    private $purchasePrice;

    /**
     * @var int $productCount
     */
    private $productCount;

    /**
     * @var Discount[] $discounts
     */
    private $discounts = array();

    /**
     * @var float $weight
     */
    private $weight;

    /**
     * @var string
     */
    private $variantName;

    /**
     * @var string
     */
    private $productName;

    /**
     * @var Category[]
     */
    private $categories = array();

    /**
     * @var Label[]
     */
    private $labels = array();

    /**
     * @return string
     */
    public function getOrderId() {
        return $this->orderId;
    }

    /**
     * @param string $orderId
     * @return OrderItem
     */
    public function setOrderId($orderId) {
        $this->orderId = $orderId;
        return $this;
    }

    /**
     * @return string
     */
    public function getProductId() {
        return $this->productId;
    }

    /**
     * @param string $productId
     * @return OrderItem
     */
    public function setProductId($productId) {
        $this->productId = $productId;
        return $this;
    }

    /**
     * @return string
     */
    public function getVariantId() {
        return $this->variantId;
    }

    /**
     * @param string $variantId
     * @return OrderItem
     */
    public function setVariantId($variantId) {
        $this->variantId = $variantId;
        return $this;
    }

    /**
     * @return Price
     */
    public function getPrice() {
        if (is_null($this->price)) {
            $this->setPrice(new Price());
        }

        return $this->price;
    }

    /**
     * @param Price $price
     * @return OrderItem
     */
    public function setPrice(Price $price) {
        $this->price = $price;
        return $this;
    }

    /**
     * @return Price
     */
    public function getPurchasePrice() {
        if (is_null($this->purchasePrice)) {
            $this->setPurchasePrice(new Price());
        }

        return $this->purchasePrice;
    }

    /**
     * @param Price $purchasePrice
     * @return OrderItem
     */
    public function setPurchasePrice(Price $purchasePrice) {
        $this->purchasePrice = $purchasePrice;
        return $this;
    }

    /**
     * @return int
     */
    public function getProductCount() {
        return $this->productCount;
    }

    /**
     * @param int $productCount
     * @return OrderItem
     */
    public function setProductCount($productCount) {
        $this->productCount = $productCount;
        return $this;
    }

    /**
     * @return Discount[]
     */
    public function getDiscounts() {
        return $this->discounts;
    }

    /**
     * @param array $discounts
     * @return $this
     */
    public function setDiscounts($discounts){
        $this->discounts[] = $discounts;
        return $this;
    }

    /**
     * @param Discount $discount
     * @return OrderItem
     */
    public function addDiscount(Discount $discount) {
        $this->discounts[] = $discount;
        return $this;
    }

    /**
     * @return float
     */
    public function getWeight() {
        return $this->weight;
    }

    /**
     * @param float $weight
     * @return OrderItem
     */
    public function setWeight($weight) {
        $this->weight = $weight;
        return $this;
    }

    /**
     * @return string
     */
    public function getVariantName(){
        return $this->variantName;
    }

    /**
     * @param string $variantName
     * @return OrderItem
     */
    public function setVariantName($variantName){
        $this->variantName = $variantName;
        return $this;
    }

    /**
     * @return string
     */
    public function getProductName(){
        return $this->productName;
    }

    /**
     * @param string $productName
     * @return OrderItem
     */
    public function setProductName($productName){
        $this->productName = $productName;
        return $this;
    }

    /**
     * @return Category[]
     */
    public function getCategories(){
        return $this->categories;
    }

    /**
     * @param Category[] $categories
     */
    public function setCategories($categories){
        $this->categories = $categories;
    }

    /**
     * @return Label[]
     */
    public function getLabels(){
        return $this->labels;
    }

    /**
     * @param Label[] $labels
     * @return OrderItem
     */
    public function setLabels($labels){
        $this->labels = $labels;
        return $this;
    }



}