<?php

namespace Monkey\ImportEshopDataObjects\Entity\Simple;


use Monkey\ImportEshopDataObjects\Base\ADifference;
use Monkey\ImportEshopDataObjects\Exceptions\UnknownOperatorException;

/**
 * Class Condition
 * @package Monkey\ImportEshopDataObjects\Entity\Simple
 */
class Condition extends ADifference {
    const OPERATOR_WHERE = " ";//space to use for first condition after WHERE
    const OPERATOR_AND = "AND";
    const OPERATOR_OR = "OR";
    const OPERATOR_XOR = "XOR";

    const COMPARE_OPERATOR_EQUALS = "=";
    const COMPARE_OPERATOR_NOT_EQUALS = "!=";
    const COMPARE_OPERATOR_DIFFERENT = "<>";
    const COMPARE_OPERATOR_BIGGER = ">";
    const COMPARE_OPERATOR_SMALLER = "<";
    const COMPARE_OPERATOR_BIGGER_EQUALS = ">=";
    const COMPARE_OPERATOR_SMALLER_EQUALS = "<=";

    private $ALLOWED_OPERATORS = array(self::OPERATOR_WHERE, self::OPERATOR_AND, self::OPERATOR_OR, self::OPERATOR_XOR);
    private $ALLOWED_COMPARE_OPERATORS = array(self::COMPARE_OPERATOR_EQUALS,
        self::COMPARE_OPERATOR_EQUALS,
        self::COMPARE_OPERATOR_NOT_EQUALS,
        self::COMPARE_OPERATOR_DIFFERENT,
        self::COMPARE_OPERATOR_BIGGER,
        self::COMPARE_OPERATOR_SMALLER,
        self::COMPARE_OPERATOR_BIGGER_EQUALS,
        self::COMPARE_OPERATOR_SMALLER_EQUALS
    );

    /**
     * @var string $operator
     */
    private $operator;

    /**
     * @var string $one
     */
    private $one;

    /**
     * @var string $compareOperator
     */
    private $compareOperator;

    /**
     * @var string $two
     */
    private $two;

    /**
     * @var string
     */
    private $raw;

    /**
     * Condition constructor.
     * @param string $operator
     * @param string $one
     * @param string $compareOperator
     * @param string $two
     */
    public function __construct($operator, $one, $compareOperator, $two) {
        $this->setOperator($operator);
        $this->setOne($one);
        $this->setCompareOperator($compareOperator);
        $this->setTwo($two);
    }

    /**
     * @return string
     */
    public function getOperator() {
        return $this->operator;
    }

    /**
     * @param string $operator
     * @return Condition
     * @throws UnknownOperatorException
     */
    public function setOperator($operator) {
        if (!in_array($operator, $this->ALLOWED_OPERATORS)) {
            throw new UnknownOperatorException($operator, $this->ALLOWED_OPERATORS);
        }

        $this->operator = $operator;
        return $this;
    }

    /**
     * @return string
     */
    public function getOne() {
        return $this->one;
    }

    /**
     * @param string $one
     * @return Condition
     */
    public function setOne($one) {
        $this->one = $one;
        return $this;
    }

    /**
     * @return string
     */
    public function getCompareOperator() {
        return $this->compareOperator;
    }

    /**
     * @param string $compareOperator
     * @return Condition
     * @throws UnknownOperatorException
     */
    public function setCompareOperator($compareOperator) {
        if (!in_array($compareOperator, $this->ALLOWED_COMPARE_OPERATORS)) {
            throw new UnknownOperatorException($compareOperator, $this->ALLOWED_COMPARE_OPERATORS);
        }

        $this->compareOperator = $compareOperator;
        return $this;
    }

    /**
     * @return string
     */
    public function getTwo() {
        return $this->two;
    }

    /**
     * @param string $two
     * @return Condition
     */
    public function setTwo($two) {
        $this->two = $two;
        return $this;
    }

    /**
     * @return string
     */
    public function getRaw(){
        return $this->raw;
    }

    /**
     * @param string $raw
     * @return Condition
     */
    public function setRaw($raw){
        $this->raw = $raw;
        return $this;
    }

    /**
     * @return string
     */
    public function __toString() {
        if (!is_null($this->getRaw())) {
            return $this->getRaw();
        }

        return " {$this->getOperator()} {$this->getOne()} {$this->getCompareOperator()} {$this->getTwo()}";
    }
}