<?php

namespace Monkey\ImportEshopDataObjects\Entity\Simple;


/**
 * Class Join
 * @package Monkey\ImportEshopDataObjects\Entity\Simple
 * @author Fingarfae
 */
class Join {
    const LEFT = "left";
    const INNER = "inner";

    /**
     * @var string
     */
    private $tableName;

    /**
     * @var string
     */
    private $alias;

    /**
     * @var Condition[]
     */
    private $conditions = array();

    /**
     * @var string
     */
    private $type = self::INNER;

    /**
     * Join constructor.
     * @param string $tableName
     * @param string $alias
     * @param Condition[] $conditions
     * @param string $type
     */
    public function __construct($tableName, $alias, array $conditions = array(), $type = self::INNER) {
        $this->tableName = $tableName;
        $this->alias = $alias;
        $this->conditions = $conditions;
        $this->type = $type;
    }

    /**
     * @return string
     */
    public function getType() {
        return $this->type;
    }

    /**
     * @param string $type
     * @return Join
     */
    public function setType($type) {
        $this->type = $type;
        return $this;
    }

    /**
     * @return string
     */
    public function getTableName() {
        return $this->tableName;
    }

    /**
     * @param string $tableName
     * @return Join
     */
    public function setTableName($tableName) {
        $this->tableName = $tableName;
        return $this;
    }

    /**
     * @return string
     */
    public function getAlias() {
        return $this->alias;
    }

    /**
     * @param string $alias
     * @return Join
     */
    public function setAlias($alias) {
        $this->alias = $alias;
        return $this;
    }

    /**
     * @return Condition[]
     */
    public function getConditions() {
        return $this->conditions;
    }

    /**
     * @param Condition[] $conditions
     * @return Join
     */
    public function setConditions($conditions) {
        $this->conditions = $conditions;
        return $this;
    }

    /**
     * @param string $operator
     * @param string $one
     * @param string $compareOperator
     * @param string $two
     * @return Condition
     */
    public function addNewCondition($operator, $one, $compareOperator, $two) {
        $condition = new Condition($operator, $one, $compareOperator, $two);
        $this->conditions[] = $condition;
        return $condition;
    }

    /**
     * @return bool
     */
    public function hasConditions() {
        return !empty($this->conditions);
    }
}