<?php

namespace Monkey\ImportEshopDataObjects\Entity;


use Monkey\ImportEshopDataObjects\Base\Entity;
use Monkey\ImportEshopDataObjects\Entity\Simple\Dimension;
use Monkey\ImportEshopDataObjects\Entity\Simple\Price;

/**
 * Class Variant
 * @package Monkey\ImportEshopDataObjects\Entities
 */
class Variant extends Entity {
    /**
     * @var string $productId
     */
    private $productId;

    /**
     * @var string $name
     */
    private $name;

    /**
     * @var string $name
     */
    private $productName;

    /**
     * @var Price $price
     */
    private $price;

    /**
     * @var Price $purchasePrice
     */
    private $purchasePrice;

    /**
     * @var string $brandId
     */
    private $brandId;

    /**
     * @var string $distributorId
     */
    private $distributorId;

    /**
     * @var float $weight
     */
    private $weight;

    /**
     * @var string $SKU
     */
    private $SKU;

    /**
     * @var string $EAN
     */
    private $EAN;

    /**
     * @var string $articleCode
     */
    private $articleCode;

    /**
     * @var string $imageSrc
     */
    private $imageSrc;

    /**
     * @var string $imageThumb
     */
    private $imageThumb;

    /**
     * @var string $unit
     */
    private $unit;

    /**
     * @var Price $unitPrice
     */
    private $unitPrice;

    /**
     * @var int $stockLevel
     */
    private $stockLevel;

    /**
     * @var Dimension[] $attributeValues
     */
    private $attributeValues = array();

    /**
     * @return string
     */
    public function getProductId() {
        return $this->productId;
    }

    /**
     * @param string $productId
     * @return Variant
     */
    public function setProductId($productId) {
        $this->productId = $productId;
        return $this;
    }

    /**
     * @return string
     */
    public function getName() {
        return $this->name;
    }

    /**
     * @param string $name
     * @return Variant
     */
    public function setName($name) {
        $this->name = $name;
        return $this;
    }

    /**
     * @return Price
     */
    public function getPrice() {
        if (is_null($this->price)) {
            $this->setPrice(new Price());
        }

        return $this->price;
    }

    /**
     * @param Price $price
     * @return Variant
     */
    public function setPrice(Price $price) {
        $this->price = $price;
        return $this;
    }

    /**
     * @return Price
     */
    public function getPurchasePrice() {
        if (is_null($this->purchasePrice)) {
            $this->setPurchasePrice(new Price());
        }

        return $this->purchasePrice;
    }

    /**
     * @param Price $purchasePrice
     * @return Variant
     */
    public function setPurchasePrice(Price $purchasePrice) {
        $this->purchasePrice = $purchasePrice;
        return $this;
    }

    /**
     * @return string
     */
    public function getBrandId() {
        return $this->brandId;
    }

    /**
     * @param string $brandId
     * @return Variant
     */
    public function setBrandId($brandId) {
        $this->brandId = $brandId;
        return $this;
    }

    /**
     * @return string
     */
    public function getDistributorId() {
        return $this->distributorId;
    }

    /**
     * @param string $distributorId
     * @return Variant
     */
    public function setDistributorId($distributorId) {
        $this->distributorId = $distributorId;
        return $this;
    }

    /**
     * @return float
     */
    public function getWeight() {
        return $this->weight;
    }

    /**
     * @param float $weight
     * @return Variant
     */
    public function setWeight($weight) {
        $this->weight = $weight;
        return $this;
    }

    /**
     * @return string
     */
    public function getSKU() {
        return $this->SKU;
    }

    /**
     * @param string $SKU
     * @return Variant
     */
    public function setSKU($SKU) {
        $this->SKU = $SKU;
        return $this;
    }

    /**
     * @return string
     */
    public function getEAN() {
        return $this->EAN;
    }

    /**
     * @param string $EAN
     * @return Variant
     */
    public function setEAN($EAN) {
        $this->EAN = $EAN;
        return $this;
    }

    /**
     * @return string
     */
    public function getArticleCode() {
        return $this->articleCode;
    }

    /**
     * @param string $articleCode
     * @return Variant
     */
    public function setArticleCode($articleCode) {
        $this->articleCode = $articleCode;
        return $this;
    }

    /**
     * @return string
     */
    public function getImageSrc() {
        return $this->imageSrc;
    }

    /**
     * @param string $imageSrc
     * @return Variant
     */
    public function setImageSrc($imageSrc) {
        $this->imageSrc = $imageSrc;
        return $this;
    }

    /**
     * @return string
     */
    public function getImageThumb() {
        return $this->imageThumb;
    }

    /**
     * @param string $imageThumb
     * @return Variant
     */
    public function setImageThumb($imageThumb) {
        $this->imageThumb = $imageThumb;
        return $this;
    }

    /**
     * @return string
     */
    public function getUnit() {
        return $this->unit;
    }

    /**
     * @param string $unit
     * @return Variant
     */
    public function setUnit($unit) {
        $this->unit = $unit;
        return $this;
    }

    /**
     * @return Price
     */
    public function getUnitPrice() {
        if (is_null($this->unitPrice)) {
            $this->setUnitPrice(new Price());
        }

        return $this->unitPrice;
    }

    /**
     * @param Price $unitPrice
     * @return Variant
     */
    public function setUnitPrice(Price $unitPrice) {
        $this->unitPrice = $unitPrice;
        return $this;
    }

    /**
     * @return int
     */
    public function getStockLevel() {
        return $this->stockLevel;
    }

    /**
     * @param int $stockLevel
     * @return Variant
     */
    public function setStockLevel($stockLevel) {
        $this->stockLevel = $stockLevel;
        return $this;
    }

    /**
     * @return Dimension[]
     */
    public function getAttributeValues() {
        return $this->attributeValues;
    }

    /**
     * @param Dimension[] $attributeValues
     * @return $this
     */
    public function setAttributeValues($attributeValues) {
        $this->attributeValues = $attributeValues;
        return $this;
    }

    /**
     * @param Dimension $attributeValue
     * @return Variant
     */
    public function addAttributeValue(Dimension $attributeValue) {
        $this->attributeValues[] = $attributeValue;
        return $this;
    }

    /**
     * @return string
     */
    public function getProductName(){
        return $this->productName;
    }

    /**
     * @param string $productName
     * @return Variant
     */
    public function setProductName($productName){
        $this->productName = $productName;
        return $this;
    }


}