// boot JS - STATIC
/**
 * Class Bootstrap
 * @namespace MonkeyDataPlugins
 * @param url string
 * @param anchor HTMLElement
 */
var MonkeyDataPluginsBootstrap = function (url, anchor) {

    /**
     * 0 | 1 | 2 | 4 | 8
     * @type number
     */
    this.verbosity = 2;

    this.boot = function () {
        log("booting up...", 8);

        var script = document.createElement('script');
        script.onload = onSuccess;
        script.onerror = onError;

        script.src = url;
        document.head.appendChild(script);

        log("waiting for browser to load foreign script...", 4);
    };


    var onSuccess = function () {
        log("foreign script loaded!", 4);

        if (typeof MonkeyDataPluginsIframe === 'undefined' || typeof MonkeyDataPluginsIframe.run !== 'function') {
            error('After loading foreign script the MonkeyDataPluginsIframe is still undefined!');
            return;
        }
        log('booted successfully', 1);

        MonkeyDataPluginsIframe.run(anchor);
    };

    var onError = function () {
        error("Unable to load foreign script");
    };

    var log = function (message, verbosityLevel) {
        if (verbosityLevel === undefined) {
            verbosityLevel = 2;
        }
        if (verbosityLevel === 0) {
            return;
        }

        if (this.verbosity >= verbosityLevel) {
            console.log(message);
        }
    };

    var error = function (error) {
        if (this.verbosity > 0) {
            console.error(error);
        }
    };

    return this;
};

console.error = function() {};
console.log = function() {};
console.warn = function() {};
console.info = function() {};