<?php

namespace Monkey\InsideBootJs\Base;

use Monkey\InsideBootJs\Exceptions\EmptyAnchorElementIdException;
use Monkey\InsideBootJs\Exceptions\FileContentGettingException;
use Monkey\InsideBootJs\Interfaces\IInsideBootJs;
use Monkey\InsidePlatformData\InsidePlatformData;
use Monkey\InsidePlatformData\Interfaces\IInsidePlatformData;

/**
 * Class AInsideBootJs
 * @package Monkey\InsideBootJs\Base
 */
abstract class AInsideBootJs implements IInsideBootJs {

    /**
     * @var string
     */
    private $anchorElementId = 'MonkeyDataPluginsAnchor';

    /**
     * @var IInsidePlatformData
     */
    private $platformData;

    /**
     * @var string
     */
    private $pluginIframeJSUrl;

    /**
     * AInsideBootJs constructor.
     * @param $pluginIframeJSUrl
     */
    public function __construct($pluginIframeJSUrl) {
        $this->pluginIframeJSUrl = $pluginIframeJSUrl;
    }

    /**
     * @return string
     */
    final protected function getPluginsIframeJSUrl() {
        return $this->pluginIframeJSUrl;
    }

    /**
     * @return string
     * @throws EmptyAnchorElementIdException
     * @throws FileContentGettingException
     */
    public function generateBootJS() {
        $js = $this->getBootJS();
        return $this->generateScriptElement($js);
    }

    /**
     * @param $js
     * @return mixed
     */
    protected function generateScriptElement($js){
        $path = array('html', 'script-element.html');
        $filePath = $this->getAssetsPath($path);
        $scriptElement = $this->getAssetsFileContent($filePath);
        return str_replace("[[JS]]", $js, $scriptElement);
    }

    /**
     * @return string
     * @throws EmptyAnchorElementIdException
     * @throws FileContentGettingException
     */
    private function getBootJS() {
        $js = '';

//        $js .= $this->getPlatformData()->generatePlatformDataJS();

        $js .= "var monkeyDataPluginsElementName = '{$this->getAnchorElementId()}';";
        $js .= "var monkeyDataPluginsUrl = '{$this->getPluginsIframeJSUrl()}';";

        $js .= $this->getBootJSGrub();
        $js .= $this->getBootJSCore();

        return $js;
    }

    /**
     * @param array $path
     * @return string
     */
    private function getAssetsPath($path = array()) {
        $assetsPath = array(__DIR__, '..', '..', 'assets');

        if (!is_array($path)) {
            $path = (array)$path;
        }

        $array = array_merge($assetsPath, $path);
        return implode(DIRECTORY_SEPARATOR, $array);
    }

    /**
     * @return string
     * @throws FileContentGettingException
     */
    protected function getBootJSGrub() {
        $path = array('js', 'boot-js-grub.js');
        $filePath = $this->getAssetsPath($path);
        return $this->getAssetsFileContent($filePath);
    }

    /**
     * @return string
     * @throws FileContentGettingException
     */
    protected function getBootJSCore() {
        $path = array('js', 'boot-js-core.js');
        $filePath = $this->getAssetsPath($path);
        return $this->getAssetsFileContent($filePath);
    }

    /**
     * @param $filePath
     * @return string
     * @throws FileContentGettingException
     */
    private function getAssetsFileContent($filePath) {
        $content = file_get_contents($filePath);
        if ($content === false) {
            throw new FileContentGettingException("File path: {$filePath}");
        }
        return (string)$content;
    }

    /**
     * @return string
     * @throws EmptyAnchorElementIdException
     */
    final public function generateAnchorElement() {
        $path = array('html', 'anchor-element-div.html');
        $filePath = $this->getAssetsPath($path);
        $anchorDivElement = $this->getAssetsFileContent($filePath);
        return str_replace("[[DIV_ID]]", $this->getAnchorElementId(), $anchorDivElement);
    }

    /**
     * @return string
     * @throws EmptyAnchorElementIdException
     */
    protected function getAnchorElementId() {
        if (empty($this->anchorElementId)) {
            throw new EmptyAnchorElementIdException();
        }
        return $this->anchorElementId;
    }

    /**
     * @param string $anchorElementId
     * @return $this
     */
    protected function setAnchorElementId($anchorElementId) {
        $this->anchorElementId = $anchorElementId;
        return $this;
    }

    /**
     * @return IInsidePlatformData
     */
    final public function getPlatformData() {
        if ($this->platformData === null) {
            $this->platformData = new InsidePlatformData();
        }
        return $this->platformData;
    }

}
