<?php

namespace Monkey\InsideBootJs\Base;

use Monkey\InsideAuth\Exceptions\InsideAuthException;
use Monkey\InsideAuth\Exceptions\InsideAuthUnauthorizedException;
use Monkey\InsideAuth\InsideAuth;
use Monkey\InsideAuth\Interfaces\IInsideAuth;
use Monkey\InsideBootJs\InsideConnectBootJs;
use Monkey\InsideBootJs\InsideIframeBootJs;
use Monkey\InsideBootJs\Interfaces\IInsideBootJs;

/**
 * Class AInsideContentController
 * @package Monkey\InsideBootJs\Base
 */
abstract class AInsideContentController extends AInsideController {

    /**
     * @return string
     */
    private function getPluginsIframeJSUrl() {
        $parts = $this->getBaseJSUrlParts();
        $parts[] = 'iframe';
        $parts[] = $this->getModel()->getIframeToken()->getAccessToken();

        return implode('/', $parts);
    }

    /**
     * @return string
     */
    private function getPluginsConnectJSUrl() {
        $parts = $this->getBaseJSUrlParts();
        $parts[] = 'connect';

        return implode('/', $parts);
    }

    /**
     * @return array
     */
    private function getBaseJSUrlParts() {
        $parts = $this->getBaseUrlParts();
        $parts[] = 'js';

        return $parts;
    }

    /**
     * @param InsideRequest $insideRequest
     * @return IInsideBootJs
     * @throws InsideAuthException
     */
    final public function handle(InsideRequest $insideRequest) {
        // connect vs connection
        if ($this->getModel()->getSecretToken() === null) {
            if ($insideRequest->getAuthCode() !== null) {
                return $this->authCode($insideRequest);
            }
            return $this->connectViaJS();
        }

        if ($this->getModel()->getIframeToken()->isExpired()) {
            try {
                return $this->refreshIframeToken();
            } catch (InsideAuthUnauthorizedException $exception){
                $this->getModel()->deleteSecretToken();
                return $this->connectViaJS();
            }
        }

        return $this->content();
    }

    /**
     * @param InsideRequest $insideRequest
     * @param int $count
     * @return IInsideBootJs
     * @throws InsideAuthException
     */
    private function authCode(InsideRequest $insideRequest, $count = 0) {
        $insideAuth = new InsideAuth($this->getConfig());

        try {
            $secretToken = $insideAuth->exchangeAuthCodeForSecretToken($insideRequest->getAuthCode());
            $this->getModel()->setSecretToken($secretToken);

        } catch (InsideAuthException $e) {
            if ($count < 3) {
                usleep(IInsideAuth::COMMUNICATION_DELAY);
                return $this->authCode($insideRequest, $count + 1);
            }
            throw $e;
        }

        $insideRequest = new InsideRequest();
        return $this->handle($insideRequest);
    }

    /**
     * @param int $count
     * @return IInsideBootJs
     * @throws InsideAuthException
     */
    private function refreshIframeToken($count = 0) {
        $insideAuth = new InsideAuth($this->getConfig());

        $secretToken = $this->getModel()->getSecretToken();
        try {
            $iframeToken = $insideAuth->exchangeSecretTokenForIframeToken($secretToken);
            $this->getModel()->setIframeToken($iframeToken);

        } catch (InsideAuthException $e) {
            if ($count < 3) {
                usleep(IInsideAuth::COMMUNICATION_DELAY);
                return $this->refreshIframeToken($count + 1);
            }
            throw $e;
        }

        $insideRequest = new InsideRequest();
        return $this->handle($insideRequest);
    }

    /**
     * @return IInsideBootJs
     */
    private function connectViaJS() {
        return new InsideConnectBootJs($this->getPluginsConnectJSUrl(), $this->getModel());
    }

    /**
     * @return IInsideBootJs
     */
    private function content() {
        return new InsideIframeBootJs($this->getPluginsIframeJSUrl());
    }

}
