<?php

namespace Monkey\InsideBootJs\Base;

use Monkey\Curl\Curl;
use Monkey\Curl\Exceptions\HttpCurlException;
use Monkey\InsideBootJs\Exceptions\InstallException;
use Monkey\InsideBootJs\Exceptions\InstallSecretTokenMissingException;

/**
 * Class AInsideInstallController
 * @package Monkey\InsideBootJs\Base
 */
abstract class AInsideInstallController extends AInsideController {

    /**
     * Uninstallation routine
     */
    final public function uninstall() {
        // if user did not finished connecting the plugin, there's no reason for remote uninstallation
        $secretToken = $this->getModel()->getSecretToken();
        if (empty($secretToken)) {
            return;
        }
        $secretTokenString = $secretToken->getAccessToken();
        if (empty($secretTokenString)) {
            return;
        }
        $url = $this->getUninstallUrl($secretTokenString);

        $parameters = array (
            'store_identity_hash' => $this->getModel()->getStoreIdentityHash()
        );

        $httpClient = new Curl();
        try {
            // we don't care about the result
            $httpClient->call($url, $parameters);
        } catch (HttpCurlException $exception) {
        }
    }



    /**
     * @throws InstallException
     * @throws InstallSecretTokenMissingException
     */
    final public function install() {

        $secretToken = $this->getModel()->getSecretToken();
        if (empty($secretToken)) {
            throw new InstallSecretTokenMissingException();
        }

        $secretTokenString = $secretToken->getAccessToken();
        if (empty($secretTokenString)) {
            throw new InstallSecretTokenMissingException();
        }

        $url = $this->getInstallUrl($secretTokenString);
        $data = $this->getModel()->getConnectionData()->jsonSerialize();

        $parameters = array (
            'data' => base64_encode(json_encode($data))
        );

        $httpClient = new Curl();
        try {
            $result = $httpClient->call($url, $parameters);
            switch ($result->getStatusCode()){
                case 401:
                    // TODO 401
                    break;
            }
        } catch (HttpCurlException $exception) {
            throw new InstallException($exception);
        }

    }


    /**
     * @param string $secretTokenString
     * @return string
     */
    private function getInstallUrl($secretTokenString) {
        $parts = $this->getBaseUrlParts();
        $parts[] = 'connection';
        $parts[] = 'install';
        $parts[] = 'backend';
        $parts[] = $secretTokenString;
        return implode('/', $parts);
    }

    /**
     * @param string $secretTokenString
     * @return string
     */
    private function getUninstallUrl($secretTokenString) {
        $parts = $this->getBaseUrlParts();
        $parts[] = 'connection';
        $parts[] = 'uninstall';
        $parts[] = $secretTokenString;
        return implode('/', $parts);
    }
}

