<?php

namespace Monkey\InsideBootJs\Model;

use Monkey\Oauth2Token\AccessToken;
use Monkey\Oauth2Token\Interfaces\IAccessToken;

/**
 * Class AModel
 * @package Monkey\InsideBootJs\Model
 */
abstract class AModel {

    /**
     * @return IAccessToken|null
     */
    public function getSecretToken() {
        $secretToken = $this->loadSecretToken();
        return $secretToken;
    }

    /**
     * @param IAccessToken $secretToken
     */
    public function setSecretToken(IAccessToken $secretToken) {
        return $this->saveSecretToken($secretToken);
    }

    /**
     * Method should delete current secret token
     * @return void
     */
    abstract public function deleteSecretToken();

    /**
     * @return IAccessToken|null
     */
    abstract protected function loadSecretToken();

    /**
     * @param IAccessToken $secretToken
     */
    abstract protected function saveSecretToken(IAccessToken $secretToken);

    /**
     * @return IAccessToken|null
     */
    public function getIframeToken() {
        $iframeToken = $this->loadIframeToken();
        if ($iframeToken === null) {
            return new AccessToken();
        }
        return $iframeToken;
    }

    /**
     * @param IAccessToken $iframeToken
     */
    public function setIframeToken(IAccessToken $iframeToken) {
        return $this->saveIframeToken($iframeToken);
    }

    /**
     * @return IAccessToken|null
     */
    abstract protected function loadIframeToken();

    /**
     * @param IAccessToken $iframeToken
     */
    abstract protected function saveIframeToken(IAccessToken $iframeToken);


    /**
     * @return \Monkey\InsidePlatformData\Interfaces\IInsideConnectionStructure
     */
    public function getConnectionData() {
        $connectionData = $this->loadConnectionData();
        return $connectionData;
    }

    /**
     * @return \Monkey\InsidePlatformData\Interfaces\IInsideConnectionStructure
     */
    abstract protected function loadConnectionData();

    /**
     * @return string
     */
    public function getStoreIdentityHash() {
        $hash = $this->loadStoreIdentityHash();
        return $hash;
    }

    /**
     * @return string
     */
    abstract protected function loadStoreIdentityHash();

}
