<?php

namespace Monkey\InsideSentry;

use Monkey\InsideSentry\Base\AInsideSentry;
use Monkey\InsideSentry\Interfaces\IRaven;

/**
 * Class InsideSentry
 * @package Monkey\InsideSentry
 */
class InsideSentry extends AInsideSentry {

    /**
     * @var IRaven $RavenClient
     */
    private static $RavenClient = null;


    /**
     *
     * @return RavenClient
     */
    public static function getRavenClient() {
        if (self::$RavenClient === null) {
            $raven = new RavenClient();
//            $raven->install();
            self::$RavenClient = $raven;
        }
        return self::$RavenClient;
    }

    /**
     * @param IRaven $ravenClient
     */
    public static function setRavenClient(IRaven $ravenClient) {
        self::$RavenClient = $ravenClient;
    }


    /**
     * Log a message to Sentry.
     *
     * @param mixed $level
     * @param string $message
     * @param array $context
     * @return boolean
     */
    public static function log($level, $message, array $context = []) {
        return static::getRavenClient()->log($level, $message, $context);
    }

    /**
     *
     * Log debug message to Sentry.
     * Level 1
     *
     * @param mixed $message
     * @param array $context
     * @return boolean
     */
    public static function debug($message, array $context = []) {
        return static::getRavenClient()->debug($message, $context);
    }

    /**
     *
     * Log info message to Sentry.
     * Level 2
     *
     * @param mixed $message
     * @param array $context
     * @return boolean
     */
    public static function info($message, array $context = []) {
        return static::getRavenClient()->info($message, $context);
    }

    /**
     *
     * Log warning message to Sentry.
     * Level 3
     *
     * @param mixed $message
     * @param array $context
     * @return boolean
     */
    public static function warning($message, array $context = []) {
        return static::getRavenClient()->warning($message, $context);
    }

    /**
     *
     * Log error message to Sentry.
     * Level 4
     *
     * @param mixed $message
     * @param array $context
     * @return boolean
     */
    public static function error($message, array $context = []) {
        return static::getRavenClient()->error($message, $context);
    }

    /**
     *
     * Log fatal message to Sentry.
     * Level 5
     *
     * @param mixed $message
     * @param array $context
     * @return boolean
     */
    public static function fatal($message, array $context = []) {
        return static::getRavenClient()->fatal($message, $context);
    }


}
