<?php

namespace Monkey\Json;

use Monkey\Json\Interfaces\IJsonSerializable;
use JsonSerializable;
use ReflectionException;

/**
 * Class JsonSerializer
 * @package Monkey\Json
 */
class JsonSerializer implements IJsonSerializable {

    /**
     * @var JsonSerializable
     */
    private $object;

    /**
     * JsonSerializer constructor.
     * @param JsonSerializable $object
     */
    public function __construct(JsonSerializable $object) {
        $this->object = $object;
    }

    /**
     * @param JsonSerializable $object
     * @return array
     */
    public static function serialize(JsonSerializable $object) {
        $instance = new static($object);
        return $instance->jsonSerialize();
    }

    /**
     * @param JsonSerializable $object
     * @return string
     */
    public static function json(JsonSerializable $object) {
        $instance = new static($object);
        return $instance->toJSON();
    }

    /**
     * @return array
     */
    public function jsonSerialize() {
        return $this->getObjectProperties($this->getObject());
    }

    /**
     * @return string
     */
    final public function toJSON() {
        return json_encode($this->jsonSerialize());
    }

    /**
     * @param JsonSerializable $object
     *
     * @return array
     */
    private function getObjectProperties(JsonSerializable $object) {
        $target = array();
        $sourceReflection = new \ReflectionObject($object);
        do {
            try {
                $sourceProperties = $sourceReflection->getProperties(\ReflectionProperty::IS_PRIVATE);
                foreach ($sourceProperties as $sourceProperty) {
                    try {
                        $sourceProperty->setAccessible(true);
                        $name = $sourceProperty->getName();

                        $attrName = $name;

                        if (array_key_exists($attrName, $target)) {
                            continue; // parent must NOT overwrite children
                        }

                        $methodName = 'get' . ucfirst($name);

                        // if exists getter use them
                        if ($sourceReflection->hasMethod($methodName)) {
                            $value = $object->$methodName();
                        } else {
                            $value = $sourceProperty->getValue($object);
                        }

                        if ($value instanceof JsonSerializable) {
                            $target[$attrName] = $this->getObjectProperties($value);
                        } else if (is_array($value)) {

                            foreach ($value as $key => $item) {
                                if ($item instanceof JsonSerializable) {
                                    $value[$key] = $this->getObjectProperties($item);
                                }
                            }
                            $target[$attrName] = $value;

                        } else {
                            $target[$attrName] = $value;
                        }

                    } catch (ReflectionException $e) {
                    }
                }
            } catch (ReflectionException $e) {
            }
        } while ($sourceReflection = $sourceReflection->getParentClass());

        return $target;
    }

    /**
     * @return JsonSerializable
     */
    private function getObject() {
        return $this->object;
    }

}
