<?php

namespace Monkey\Oauth2Token\Base;

use DateTime;
use Monkey\Json\Base\AJson;
use Monkey\Oauth2Token\Interfaces\IToken;

/**
 * Class AToken
 * @package Monkey\Oauth2Token\Base
 */
abstract class AToken extends AJson implements IToken {

    /**
     *
     * @var int
     */
    private $createdAt;

    /**
     *
     * @var int
     */
    private $expiresAt;

    /**
     * @param int $timestamp
     *
     * @return DateTime
     */
    final protected function getDateTimeFromTimestamp($timestamp) {
        $dateTimeHelper = $this->getNewDateTime();
        $dateTimeHelper->setTimestamp($timestamp);
        return $dateTimeHelper;
    }

    /**
     * @return DateTime
     */
    final protected function getNewDateTime() {
        $dth = new DateTime();
        return $dth;
    }

    /**
     * @return DateTime
     */
    final public function getCreatedAt() {
        return $this->getDateTimeFromTimestamp($this->createdAt);
    }

    /**
     * @param int $createdAt
     * @return $this
     */
    final public function setCreatedAt($createdAt) {
        $this->createdAt = $createdAt;
        return $this;
    }

    /**
     * @return DateTime
     */
    final public function getExpiresAt() {
        return $this->getDateTimeFromTimestamp($this->expiresAt);
    }

    /**
     * @param int $expiresAt
     * @return $this
     */
    final public function setExpiresAt($expiresAt) {
        $this->expiresAt = $expiresAt;
        return $this;
    }

    /**
     * @return bool
     */
    final public function isExpired() {
        return ($this->expiresAt < time());
    }

    /**
     * @return int
     */
    final public function getExpiresIn() {
        return $this->getExpiresAt()->getTimestamp() - time();
    }

    /**
     * @return array
     */
    public function jsonSerialize() {
        $array = parent::jsonSerialize();

        $array['createdAt'] = $this->getCreatedAt()->getTimestamp();
        $array['expiresAt'] = $this->getExpiresAt()->getTimestamp();

        ksort($array);

        return $array;
    }

}
