<?php

namespace Monkey\Oauth2Token;

use Monkey\Oauth2Token\Base\AToken;
use Monkey\Oauth2Token\Interfaces\IAccessToken;
use Monkey\Oauth2Token\Interfaces\IRefreshToken;
use Monkey\Oauth2Token\Interfaces\IScope;
use stdClass;

/**
 * Class RefreshToken
 * @package Monkey\Oauth2Token
 */
class RefreshToken extends AToken implements IRefreshToken {

    /**
     * @var IAccessToken
     */
    private $accessToken;

    /**
     *
     * @var string
     */
    private $refreshToken;

    /**
     * @param IAccessToken $accessToken
     */
    public function __construct(IAccessToken $accessToken) {
        $this->accessToken = $accessToken;
    }

    /**
     * @return IAccessToken
     */
    protected function getAccessToken() {
        return $this->accessToken;
    }

    /**
     * @return string
     */
    final public function getRefreshToken() {
        return $this->refreshToken;
    }

    /**
     * @param string $refreshToken
     * @return $this
     */
    final public function setRefreshToken($refreshToken) {
        $this->refreshToken = $refreshToken;
        return $this;
    }

    /**
     * @return IScope[]
     */
    final public function getScopes() {
        return $this->getAccessToken()->getScopes();
    }

    /**
     * @param string $code
     * @return bool
     */
    final public function hasScopeByCode($code) {
        return $this->getAccessToken()->hasScopeByCode($code);
    }

    /**
     * @return array
     */
    public function jsonSerialize() {
        $array = parent::jsonSerialize();

        // no accessToken because it is cyclic reference
        unset($array['accessToken']);

        return $array;
    }

    /**
     * @param IAccessToken $accessToken
     * @param stdClass $object
     * @return static
     */
    public static function make(IAccessToken $accessToken, stdClass $object) {
        $refreshToken = new static($accessToken);

        if (isset($object->refreshToken)) {
            $refreshToken->setRefreshToken($object->refreshToken);
        }
        if (isset($object->createdAt)) {
            $refreshToken->setCreatedAt($object->createdAt);
        }
        if (isset($object->expiresAt)) {
            $refreshToken->setExpiresAt($object->expiresAt);
        }

        return $refreshToken;
    }

}
