<?php
/**
 * Created by PhpStorm.
 * User: Fingarfae
 * Date: 31. 1. 2018
 * Time: 12:23
 */

namespace Monkey\PluginsPrestashop15XApi\Base;


use Monkey\ImportEshopApi\Exceptions\DataMiningFailedException;
use Monkey\ImportEshopDataObjects\Entity\Label;
use Monkey\PluginsPrestashop15XApi\Facades\Db;
use Monkey\PluginsPrestashop15XApi\Facades\DbQuery;

class LabelHelper
{
    /**
     * @var array
     */
    private $labels;

    /**
     * @var int
     */
    private $id_lang;

    public function __construct($id_lang){
        $this->setIdLang($id_lang);
    }

    /**
     * @param int $productId
     * @return Label[]
     */
    public function getLabels($productId){
        if(is_null($this->labels)){
            $this->loadLabels();
        }
        if(array_key_exists($productId, $this->labels)){
            return $this->labels[$productId];
        }
        return array();
    }

    private function loadLabels(){
        $builder = new DbQuery();
        $builder->select("pt.id_product,t.name,t.id_tag");
        $builder->from("tag","t");
        $builder->leftJoin("product_tag", "pt","t.id_tag = pt.id_tag");
        $builder->where("t.id_lang = ".cSQL($this->getIdLang()));

        try {
            $result = Db::executeS($builder->build());
        } catch (\Exception $exception) {
            throw new DataMiningFailedException($exception);
        }

        if($result){
            foreach($result as $row){
                $label = new Label();
                $label->setId($row['id_tag']);
                $label->setName($row['name']);
                $this->labels[$row['id_product']][] = $label;
            }
        }
    }

    /**
     * @return int
     */
    protected function getIdLang(){
        return $this->id_lang;
    }

    /**
     * @param int $id_lang
     * @return LabelHelper
     */
    protected function setIdLang($id_lang){
        $this->id_lang = $id_lang;
        return $this;
    }


}