<?php

namespace Monkey\PluginsPrestashop15XApi\Endpoint;


use Exception;
use Monkey\ImportEshopApi\Endpoint\AAttributeValues;
use Monkey\ImportEshopApi\Exceptions\DataMiningFailedException;
use Monkey\ImportEshopDataObjects\Entity\Simple\Join;
use Monkey\PluginsPrestashop15XApi\Base\PrestaDefaultEshopSetting;
use Monkey\PluginsPrestashop15XApi\Facades\Db;
use Monkey\PluginsPrestashop15XApi\Facades\DbQuery;

/**
 * Class AttributeValues
 * @package Monkey\PluginsPrestashop15XApi\Endpoint
 */
class AttributeValues extends AAttributeValues {
    /**
     * @return string
     */
    protected function getComposedQuery() {
        $builder = new DbQuery();
        $this->getSelectAttributes($builder);
        $this->getTable($builder);
        $this->getConditions($builder);
        $this->getDefaultLimits($builder);
        $sql = $builder->build();
        return $sql;
    }

    /**
     * @return string
     * @throws DataMiningFailedException
     */
    protected function getTableSchema() {
        $tables = array("attribute","attribute_lang","attribute_group_lang");

        $results = array();
        foreach($tables as $tableName){
            $tableName = $this->getDefaultEshopSetting()->getDbPrefix() . $tableName;
            $sql = "SHOW COLUMNS FROM {$tableName}";

            try {
                $columns = Db::executeS($sql);
            } catch (Exception $exception) {
                throw new DataMiningFailedException($exception);
            }

            foreach($columns as &$row){
                $row['table']=$tableName;
            }

            $results = array_merge($results, $columns);
        }

        return $results;
    }

    /**
     * @throws DataMiningFailedException
     */
    protected function setupResponseData() {
        $sql = $this->getComposedQuery();

        if ($this->getRequest()->isDebug()) {
            $this->getResponse()->setQuery($sql);
            $this->getResponse()->setSchema($this->getTableSchema());
        }

        try {
            $results = Db::executeS($sql);
        } catch (Exception $exception) {
            throw new DataMiningFailedException($exception);
        }

        if ($results) {
            foreach ($results as $row) {
                $this->getResponse()->addNewEntity()
                    ->setName($row["name"])
                    ->setAttributeId($row["attribute_id"])
                    ->setAttributeName($row["attribute_name"])
                    ->setIdShop($row["shop_id"])
                    ->setId($row["id"]);
            }
        }
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultSelectAttributes($builder = null) {
        $columns = array(
            "attribute_value.id_attribute AS id",
            "attribute_value_lang.name",
            "attribute.id_attribute_group AS attribute_id",
            "attribute.name AS attribute_name",
            "attribute_value_shop.id_shop as shop_id"
        );
        $builder->select(implode(", ", $columns));
        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultTable($builder = null) {
        $builder->from("attribute", "attribute_value");
        $builder->innerJoin("attribute_lang", "attribute_value_lang", "attribute_value.id_attribute = attribute_value_lang.id_attribute");
        $builder->innerJoin("attribute_group_lang", "attribute", "attribute_value.id_attribute_group = attribute.id_attribute_group");
        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultCondition($builder = null) {
        $lang = cSQL($this->getDefaultEshopSetting()->getLanguageId());
        $builder->where("attribute.id_lang = '{$lang}'");
        $builder->where("attribute_value_lang.id_lang = '{$lang}'");

        $shopId = $this->getRequest()->getCurrentDefaultSetting("id_shop");

        if (!is_null($shopId)) {
            $builder->innerJoin("attribute_shop", "attribute_value_shop", "attribute_value.id_attribute = attribute_value_shop.id_attribute AND attribute_value_shop.id_shop = '" . cSQL($shopId->getValue()) . "'");
            $builder->innerJoin("attribute_group_shop", "attribute_shop", "attribute.id_attribute_group = attribute_shop.id_attribute_group AND attribute_shop.id_shop = '" . cSQL($shopId->getValue()) . "'");
        }

        if (!is_null($this->getRequest()->getId())) {
            $builder->where("attribute_value.id_attribute = '" . cSQL($this->getRequest()->getId()) . "'");
            return true;
        }

        if (!is_null($this->getRequest()->getForeignKey())) {
            $builder->where("attribute.id_attribute_group = '" . cSQL($this->getRequest()->getForeignKey()) . "'");
            return true;
        }

        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultLimits($builder = null) {
        $builder->limit($this->getResultsCount(), $this->getOffset());
        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentAttributes($builder = null) {
        $return = false;

        foreach ($this->getRequest()->getDifferences()->getColumns() as $column) {
            $return = true;
            $builder->select($column->__toString());
        }

        if ($return) return true;
        return null;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentTable($builder = null) {
        $return = false;

        foreach ($this->getRequest()->getDifferences()->getTableAliases() as $table) {
            $return = true;

            if ($table->hasConditions()) {
                $conditions = "";

                foreach ($table->getConditions() as $condition) {
                    $conditions .= $condition->__toString();
                }

                $conditions = $this->getRequest()->setupVariables($conditions);

                switch ($table->getType()) {
                    case Join::INNER:
                        $builder->innerJoin($table->getTableName(), $table->getAlias(), $conditions);
                        break;
                    case Join::LEFT:
                        $builder->leftJoin($table->getTableName(), $table->getAlias(), $conditions);
                        break;
                }
            } else {
                $builder->from($table->getTableName(), $table->getAlias());
            }
        }

        if ($return) return true;
        return null;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentConditions($builder = null) {
        $return = false;
        $conditions = "";

        foreach ($this->getRequest()->getDifferences()->getAdditionalConditions() as $condition) {
            $return = true;
            $conditions .= $condition->__toString();
        }

        $builder->where($this->getRequest()->setupVariables($conditions));

        if ($return) return true;
        return null;
    }

    /**
     * @return PrestaDefaultEshopSetting
     */
    protected function getDefaultEshopSetting() {
        return parent::getDefaultEshopSetting();
    }
}