<?php

namespace Monkey\PluginsPrestashop15XApi\Endpoint;


use Exception;
use Monkey\ImportEshopApi\Endpoint\ACustomers;
use Monkey\ImportEshopApi\Exceptions\DataMiningFailedException;
use Monkey\ImportEshopDataObjects\Entity\Simple\Join;
use Monkey\PluginsPrestashop15XApi\Base\PrestaDefaultEshopSetting;
use Monkey\PluginsPrestashop15XApi\Facades\Db;
use Monkey\PluginsPrestashop15XApi\Facades\DbQuery;

/**
 * Class Customers
 * @package Monkey\PluginsPrestashop15XApi\Endpoint
 */
class Customers extends ACustomers {
    /**
     * @return string
     */
    protected function getComposedQuery() {
        $builder = new DbQuery();
        $this->getSelectAttributes($builder);
        $this->getTable($builder);
        $this->getConditions($builder);
        $this->getDefaultLimits($builder);
        $sql = $builder->build();
        return $sql;
    }

    /**
     * @return string
     * @throws DataMiningFailedException
     */
    protected function getTableSchema() {

        $tables = array("customer","address","country_lang");

        $results = array();
        foreach($tables as $tableName){
            $tableName = $this->getDefaultEshopSetting()->getDbPrefix() . $tableName;
            $sql = "SHOW COLUMNS FROM {$tableName}";

            try {
                $columns = Db::executeS($sql);
            } catch (Exception $exception) {
                throw new DataMiningFailedException($exception);
            }

            foreach($columns as &$row){
                $row['table']=$tableName;
            }

            $results = array_merge($results, $columns);
        }

        return $results;
    }

    /**
     * @throws DataMiningFailedException
     */
    protected function setupResponseData() {
        $sql = $this->getComposedQuery();

        try {
            $results = Db::executeS($sql);
        } catch (Exception $exception) {
            throw new DataMiningFailedException($exception);
        }

        if ($results) {
            foreach ($results as $row) {
                $customer = $this->getResponse()->addNewEntity();
                $customer->setId($row['id_customer']);
                $customer->setIdShop($row['id_shop']);
                $customer->setIdShopGroup($row['id_shop_group']);

                $customer->setCreatedAt($row['date_add']);
                $customer->setUpdatedAt($row['date_upd']);

                $customer->setFirstname($row['firstname']);
                $customer->setLastname($row['lastname']);
                $customer->setEmail($row['email']);

                $customer->setCountry($row['country']);
                $customer->setCity($row['city']);
                $customer->setZipCode($row['postcode']);

                $customer->setRegistered($row['is_guest']);
                $customer->setCompany(!is_null($row['company']));
                $customer->setVatPayer(!is_null($row['vat_number']) AND (string)$row['vat_number'] != "");
            }
        }
    }

    /**
     * @param DbQuery|null $builder
     * @return bool
     */
    protected function getDefaultSelectAttributes($builder = null) {
        $columns = array(
            "c.id_customer","c.date_add","c.date_upd","c.firstname","c.lastname","c.email",
            "a.city","cl.name as country","a.postcode",
            "c.company","c.is_guest","a.vat_number","c.id_shop","c.id_shop_group"
        );
        $builder->select(implode(", ", $columns));
        return true;
    }

    /**
     * @param DbQuery|null $builder
     * @return bool
     */
    protected function getDefaultTable($builder = null) {
        $innerBuilder = new DbQuery();
        $innerBuilder->select("MAX(id_address) as id_address, id_customer");
        $innerBuilder->from("address");
        $innerBuilder->groupBy("id_customer");

        $builder->from("customer","c"); //jejich builder do nazvu tabulky sam pridava prefix.... v builderu pouzivat alias asi.
        $builder->joinSubQuery("(".$innerBuilder->build().")","ah","ah.id_customer = c.id_customer");
        $builder->leftJoin("address","a","ah.id_address = a.id_address");
        $builder->leftJoin("country_lang","cl","a.id_country = cl.id_country AND cl.id_lang = ".cSQL($this->getDefaultEshopSetting()->getLanguageId()));
        return true;
    }

    /**
     * @param DbQuery|null $builder
     * @return bool
     */
    protected function getDefaultCondition($builder = null) {
        $shopId = $this->getRequest()->getCurrentDefaultSetting("id_shop");
        $shopGroupId = $this->getRequest()->getCurrentDefaultSetting("id_shop_group");

        if (!is_null($shopId)) {
            $builder->where("c.id_shop = '" . cSQL($shopId->getValue()) . "'");
        }

        if (!is_null($shopGroupId)) {
            $builder->where("c.id_shop_group = ".cSQL($shopGroupId->getValue()));
        }

        if(!is_null($this->getRequest()->getId())){
            $builder->where("c.id_customer = ".cSQL($this->getRequest()->getId()));
            return true;
        }

        //Select by dates
        $createdCondition = "";
        $updatedCondition = "";
        $dateCondition = "";
        $createdFrom = $this->getRequest()->getCreatedFrom();
        $createdTo = $this->getRequest()->getCreatedTo();
        $updatedFrom = $this->getRequest()->getUpdatedFrom();
        $updatedTo = $this->getRequest()->getUpdatedTo();

        if (!is_null($createdFrom) && !is_null($createdTo)) {
            $createdCondition .= "c.date_add BETWEEN '" . cSQL($createdFrom) . "' AND '" . cSQL($createdTo) . "'";
        } else if (!is_null($createdFrom)) {
            $createdCondition .= "c.date_add >= '" . cSQL($createdFrom) . "'";
        } else if (!is_null($createdTo)) {
            $createdCondition .= "c.date_add <= '" . cSQL($createdTo) . "'";
        }

        if (!is_null($updatedFrom) && !is_null($updatedTo)) {
            $updatedCondition .= "c.date_upd BETWEEN '" . cSQL($updatedFrom) . "' AND '" . cSQL($updatedTo) . "'";
        } else if (!is_null($updatedFrom)) {
            $updatedCondition .= "c.date_upd >= '" . cSQL($updatedFrom) . "'";
        } else if (!is_null($updatedTo)) {
            $updatedCondition .= "c.date_upd <= '" . cSQL($updatedTo) . "'";
        }
        if (!empty($createdCondition) && !empty($updatedCondition)) {
            $dateCondition = $createdCondition . " OR " . $updatedCondition;
        } else {
            $dateCondition = $createdCondition . $updatedCondition;
        }

        if (!empty($dateCondition)) {
            $builder->where($dateCondition);
            return true;
        }


        $sinceId = $this->getRequest()->getSinceId();
        $maxId = $this->getRequest()->getMaxId();
        if (!is_null($sinceId) && !is_null($maxId)) {
            $builder->where("c.id_customer BETWEEN ". cSQL($sinceId+1) . " AND " . cSQL($maxId-1));
            return true;
        } else if (!is_null($sinceId)) {
            $builder->where("c.id_customer > ". cSQL($sinceId));
            return true;
        } else if (!is_null($maxId)) {
            $builder->where("c.id_customer < ". cSQL($maxId));
            return true;
        }
        return false;
    }

    /**
     * @param DbQuery|null $builder
     * @return bool
     */
    protected function getDefaultLimits($builder = null) {
        $builder->limit($this->getResultsCount(), $this->getOffset());
        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentAttributes($builder = null) {
        $return = false;

        foreach ($this->getRequest()->getDifferences()->getColumns() as $column) {
            $return = true;
            $builder->select($column->__toString());
        }

        if ($return) return true;
        return null;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentTable($builder = null) {
        $return = false;

        foreach ($this->getRequest()->getDifferences()->getTableAliases() as $table) {
            $return = true;

            if ($table->hasConditions()) {
                $conditions = "";

                foreach ($table->getConditions() as $condition) {
                    $conditions .= $condition->__toString();
                }

                $conditions = $this->getRequest()->setupVariables($conditions);

                switch ($table->getType()) {
                    case Join::INNER:
                        $builder->innerJoin($table->getTableName(), $table->getAlias(), $conditions);
                        break;
                    case Join::LEFT:
                        $builder->leftJoin($table->getTableName(), $table->getAlias(), $conditions);
                        break;
                }
            } else {
                $builder->from($table->getTableName(), $table->getAlias());
            }
        }

        if ($return) return true;
        return null;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentConditions($builder = null) {
        $return = false;
        $conditions = "";

        foreach ($this->getRequest()->getDifferences()->getAdditionalConditions() as $condition) {
            $return = true;
            $conditions .= $condition->__toString();
        }

        $builder->where($this->getRequest()->setupVariables($conditions));

        if ($return) return true;
        return null;
    }

    /**
     * @return PrestaDefaultEshopSetting
     */
    protected function getDefaultEshopSetting() {
        return parent::getDefaultEshopSetting();
    }
}