<?php
/**
 * Created by PhpStorm.
 * User: Fingarfae
 * Date: 15. 1. 2018
 * Time: 13:30
 */

namespace Monkey\PluginsPrestashop15XApi\Endpoint;


use Monkey\ImportEshopApi\Endpoint\AOrderItems;
use Monkey\ImportEshopApi\Exceptions\DataMiningFailedException;
use Monkey\ImportEshopDataObjects\Entity\Simple\Discount;
use Monkey\ImportEshopDataObjects\Entity\Simple\Join;
use Monkey\ImportEshopDataObjects\Entity\Simple\Price;
use Monkey\PluginsPrestashop15XApi\Base\CategoryHelper;
use Monkey\PluginsPrestashop15XApi\Base\LabelHelper;
use Monkey\PluginsPrestashop15XApi\Base\PrestaDefaultEshopSetting;
use Monkey\PluginsPrestashop15XApi\Facades\Db;
use Monkey\PluginsPrestashop15XApi\Facades\DbQuery;
use Exception;

class OrderItems extends AOrderItems
{

    /**
     * @return string
     */
    protected function getComposedQuery() {
        $builder = new DbQuery();
        $this->getSelectAttributes($builder);
        $this->getTable($builder);
        $this->getConditions($builder);
        $this->getDefaultLimits($builder);
        $sql = $builder->build();
        return $sql;
    }

    /**
     * @return string
     * @throws DataMiningFailedException
     */
    protected function getTableSchema() {
        $tables = array("order_detail","order_detail_tax","tax","orders","currency","product_lang");

        $results = array();
        foreach($tables as $tableName){
            $tableName = $this->getDefaultEshopSetting()->getDbPrefix() . $tableName;
            $sql = "SHOW COLUMNS FROM {$tableName}";

            try {
                $columns = Db::executeS($sql);
            } catch (Exception $exception) {
                throw new DataMiningFailedException($exception);
            }

            foreach($columns as &$row){
                $row['table']=$tableName;
            }

            $results = array_merge($results, $columns);
        }

        return $results;
    }

    /**
     * @throws DataMiningFailedException
     */
    protected function setupResponseData() {

        $sql = $this->getComposedQuery();

        try {
            $results = Db::executeS($sql);
        } catch (\Exception $exception) {
            throw new DataMiningFailedException($exception);
        }

        if ($results) {

            $categoryHelper = new CategoryHelper(
                $this->getDefaultEshopSetting()->getLanguageId(),
                $this->getRequest()->getCurrentDefaultSetting("id_shop")->getValue(),
                $this->getRequest()->getCurrentDefaultSetting("id_shop_group")->getValue()
            );

            $labelHelper = new LabelHelper($this->getDefaultEshopSetting()->getLanguageId());

            foreach ($results as $row) {
                $productPriceWithoutVat = $row["original_product_price"];
                $productPriceWithVat = $row["original_product_price"] * (($row["rate"] + 100) / 100);

                $price = new Price();
                $price->setPrice($productPriceWithVat)
                    ->setPriceWithoutVat($productPriceWithoutVat)
                    ->setCurrency($row["iso_code"])
                    ->setRate($row["rate"])
                    ->setConversionRate($row["conversion_rate"]);

                $purchasePrice = new Price();
                $purchasePrice->setRate(0);
                $purchasePrice->setCurrency($this->getDefaultEshopSetting()->getCurrencyIso());
                $purchasePriceValue = floatval($row["purchase_supplier_price"]);

                if ($purchasePriceValue == 0) {
                    $purchasePriceBuilder = new DbQuery();
                    $purchasePriceBuilder->select("IF(pas.wholesale_price = 0, ps.wholesale_price, pas.wholesale_price) AS purchase_price")
                        ->from("product_attribute_shop", "pas")
                        ->innerJoin("product_attribute", "pa", "pa.id_product_attribute = pas.id_product_attribute")
                        ->innerJoin("product_shop", "ps", "pa.id_product = ps.id_product")
                        ->where("pas.id_product_attribute = '" . cSQL($row["product_attribute_id"]) . "'")
                        ->where("pas.id_shop = '" . cSQL($row["id_shop"]) . "'")
                        ->where("ps.id_shop = '" . cSQL($row["id_shop"]) . "'");
                    $purchasePriceResult = Db::executeS($purchasePriceBuilder->build());

                    if (!empty($purchasePriceResult)) {
                        $purchasePriceValue = $purchasePriceResult[0]["purchase_price"];
                    }
                }

                $purchasePrice->setPriceWithoutVat($purchasePriceValue);
                $purchasePrice->setPrice($purchasePriceValue);

                $orderItem = $this->getResponse()->addNewEntity();
                $orderItem->setId($row["id_order_detail"]);
                $orderItem->setPrice($price);
                $orderItem->setPurchasePrice($purchasePrice);
                $orderItem->setWeight($row["product_weight"]);
                $orderItem->setVariantId($row["product_attribute_id"]);
                $orderItem->setOrderId($row["id_order"]);
                $orderItem->setProductCount($row["product_quantity"]);
                $orderItem->setProductId($row["product_id"]);
                $orderItem->setVariantName($row["product_name"]);
                $orderItem->setProductName($row["name"]);
                $orderItem->setIdShop($row["shop_id"]);

                if($productPriceWithVat > $row["unit_price_tax_incl"]){
                    //discount
                    $discountValue = $productPriceWithVat - $row["unit_price_tax_incl"];
                    $discount = new Discount();
                    $discount->setValue($discountValue);
                    $orderItem->addDiscount($discount);
                }

                $orderItem->setCategories($categoryHelper->getCategoryIds($row['product_id']));
                $orderItem->setLabels($labelHelper->getLabels($row['product_id']));
            }
        }
    }

    /**
     * @param DbQuery|null $builder
     * @return bool
     */
    protected function getDefaultSelectAttributes($builder = null) {
        $columns = array(
            "od.id_order_detail",
            "od.id_order",
            "od.product_id",
            "od.product_attribute_id",
            "od.product_name",
            "od.product_quantity",
            "od.product_weight",
            "od.unit_price_tax_incl",
            "od.unit_price_tax_excl",
            "od.purchase_supplier_price",
            "od.original_product_price",
            "t.rate",
            "pl.name",
            "od.id_shop AS shop_id",
            "o.conversion_rate",
            "c.iso_code"
        );
        $builder->select(implode(", ", $columns));
        return true;
    }

    /**
     * @param DbQuery|null $builder
     * @return bool
     */
    protected function getDefaultTable($builder = null) {
        $orderConditions = array("od.id_order = o.id_order");
        $productLangCondition = array("od.product_id = pl.id_product");
        $shopId = $this->getRequest()->getCurrentDefaultSetting("id_shop");
        if (!is_null($shopId)) {
            $orderConditions[] = "o.id_shop = '" . cSQL($shopId->getValue()) . "'";
            $productLangCondition[] = "pl.id_shop = '" . cSQL($shopId->getValue()) . "'";
        }
        $shopGroupId =  $this->getRequest()->getCurrentDefaultSetting("id_shop_group");
        if (!is_null($shopGroupId)) {
            $orderConditions[] = "o.id_shop_group = '" . cSQL($shopGroupId->getValue()) . "'";
        }
        $langId = cSQL($this->getDefaultEshopSetting()->getLanguageId());
        $productLangCondition[] = "pl.id_lang = '" . cSQL($langId) . "'";

        $builder->from("order_detail","od");
        $builder->leftJoin("order_detail_tax","odt","od.id_order_detail = odt.id_order_detail");
        $builder->leftJoin("tax","t","odt.id_tax = t.id_tax");
        $builder->leftJoin("orders","o",implode(" AND ", $orderConditions));
        $builder->leftJoin("currency","c","o.id_currency = c.id_currency");
        $builder->leftJoin("product_lang","pl",implode(" AND ",$productLangCondition));
        return true;
    }

    /**
     * @param DbQuery|null $builder
     * @return bool
     */
    protected function getDefaultCondition($builder = null) {
        $shopId = $this->getRequest()->getCurrentDefaultSetting("id_shop");
        if (!is_null($shopId)) {
            $builder->where("od.id_shop = '" . cSQL($shopId->getValue()) . "'");
        }

        if (!is_null($this->getRequest()->getForeignKey())) {
            $builder->where("od.id_order = '" . cSQL($this->getRequest()->getForeignKey()) . "'");
            return true;
        }

        if (!is_null($this->getRequest()->getId())) {
            $builder->where("od.id_order_detail = '".cSQL($this->getRequest()->getId()) . "'");
            return true;
        }

        return false;
    }

    /**
     * @param DbQuery|null $builder
     * @return bool
     */
    protected function getDefaultLimits($builder = null) {
        $builder->limit($this->getResultsCount(), $this->getOffset());
        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentAttributes($builder = null) {
        $return = false;

        foreach ($this->getRequest()->getDifferences()->getColumns() as $column) {
            $return = true;
            $builder->select($column->__toString());
        }

        if ($return) return true;
        return null;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentTable($builder = null) {
        $return = false;

        foreach ($this->getRequest()->getDifferences()->getTableAliases() as $table) {
            $return = true;

            if ($table->hasConditions()) {
                $conditions = "";

                foreach ($table->getConditions() as $condition) {
                    $conditions .= $condition->__toString();
                }

                $conditions = $this->getRequest()->setupVariables($conditions);

                switch ($table->getType()) {
                    case Join::INNER:
                        $builder->innerJoin($table->getTableName(), $table->getAlias(), $conditions);
                        break;
                    case Join::LEFT:
                        $builder->leftJoin($table->getTableName(), $table->getAlias(), $conditions);
                        break;
                }
            } else {
                $builder->from($table->getTableName(), $table->getAlias());
            }
        }

        if ($return) return true;
        return null;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentConditions($builder = null) {
        $return = false;
        $conditions = "";

        foreach ($this->getRequest()->getDifferences()->getAdditionalConditions() as $condition) {
            $return = true;
            $conditions .= $condition->__toString();
        }

        $builder->where($this->getRequest()->setupVariables($conditions));

        if ($return) return true;
        return null;
    }

    /**
     * @return PrestaDefaultEshopSetting
     */
    protected function getDefaultEshopSetting() {
        return parent::getDefaultEshopSetting();
    }


}