<?php

namespace Monkey\PluginsPrestashop15XApi\Endpoint;


use Exception;
use Monkey\ImportEshopApi\Endpoint\AVariants;
use Monkey\ImportEshopApi\Exceptions\DataMiningFailedException;
use Monkey\ImportEshopDataObjects\Entity\Simple\Dimension;
use Monkey\ImportEshopDataObjects\Entity\Simple\Join;
use Monkey\ImportEshopDataObjects\Entity\Simple\Price;
use Monkey\PluginsPrestashop15XApi\Base\PrestaDefaultEshopSetting;
use Monkey\PluginsPrestashop15XApi\Facades\Db;
use Monkey\PluginsPrestashop15XApi\Facades\DbQuery;
use Monkey\PluginsPrestashop15XApi\Facades\Link;

/**
 * Class Variants
 * @package Monkey\PluginsPrestashop15XApi\Endpoint
 * @author Fingarfae
 */
class Variants extends AVariants {
    /**
     * @return string
     */
    protected function getComposedQuery() {
        $builder = new DbQuery();
        $this->getSelectAttributes($builder);
        $this->getTable($builder);
        $this->getConditions($builder);
        $this->getDefaultLimits($builder);
        $sql = $builder->build();
        return $sql;
    }

    /**
     * @return string
     * @throws DataMiningFailedException
     */
    protected function getTableSchema() {
        $tables = array("product_attribute","product_lang","product_attribute_image","product");

        $results = array();
        foreach($tables as $tableName){
            $tableName = $this->getDefaultEshopSetting()->getDbPrefix() . $tableName;
            $sql = "SHOW COLUMNS FROM {$tableName}";

            try {
                $columns = Db::executeS($sql);
            } catch (Exception $exception) {
                throw new DataMiningFailedException($exception);
            }

            foreach($columns as &$row){
                $row['table']=$tableName;
            }

            $results = array_merge($results, $columns);
        }

        return $results;
    }

    /**
     * @throws DataMiningFailedException
     */
    protected function setupResponseData() {

        $sql = $this->getComposedQuery();

        try {
            $results = Db::executeS($sql);
        } catch (Exception $exception) {
            throw new DataMiningFailedException($exception);
        }

        if ($results) {
            foreach ($results as $row) {

                $productVariant = $this->getResponse()->addNewEntity();
                $productVariant->setIdShop($row['id_shop']);

                $productVariant->setId($row['id_product_attribute']);
                $productVariant->setCreatedAt($row['date_add']);
                $productVariant->setUpdatedAt($row['date_upd']);
                $productVariant->setImageSrc(!is_null($row['id_image'])?Link::getImageLink($row['link_rewrite'], $row['id_image'], Link::IMAGE_TYPE_LARGE):"");
                $productVariant->setImageThumb(!is_null($row['id_image'])?Link::getImageLink($row['link_rewrite'], $row['id_image'], Link::IMAGE_TYPE_SMALL):"");

                $price = new Price();
                $price->setPriceWithoutVat($row['price']);
                $price->setCurrency($this->getDefaultEshopSetting()->getCurrencyIso());


                $productVariant->setPrice($price);
                $productVariant->setProductId($row['id_product']);
                $productVariant->setWeight($row['weight']);

                $purchasePrice = new Price();
                $purchasePrice->setPriceWithoutVat($row['purchase_price']);
                $purchasePrice->setCurrency($this->getDefaultEshopSetting()->getCurrencyIso());
                $productVariant->setPurchasePrice($purchasePrice);

                $productVariant->setUnitPrice($price);
                $productVariant->setUnit($row['unity']);

                $productVariant->setArticleCode($row['reference']);
                $productVariant->setEAN($row['ean13']);
                $productVariant->setSKU($row['upc']);

                $productVariant->setBrandId($row['brand']);
                $productVariant->setDistributorId($row['distributor']);

                $productVariant->setStockLevel($row['quantity']);
                $productVariant->setAttributeValues($this->getDimensions($productVariant->getId()));

                $productVariant->setName(variantNameCompose($row['name'], $productVariant->getAttributeValues()));
                $productVariant->setProductName($row['name']);
            }
        }
    }

    /**
     * @param int $id_product_attribute
     * @return Dimension[]
     * @throws DataMiningFailedException
     */
    protected function getDimensions($id_product_attribute) {
        $builder = new DbQuery();
        $builder->select(implode(", ", array("pac.id_product_attribute", "al.id_attribute", "al.name as value",
            "agl.id_attribute_group ", "agl.name", "agl.public_name")));
        $builder->from("product_attribute_combination","pac");
        $builder->leftJoin("attribute","a","pac.id_attribute = a.id_attribute");
        $builder->leftJoin("attribute_lang","al","al.id_attribute = a.id_attribute AND al.id_lang = ".cSQL($this->getDefaultEshopSetting()->getLanguageId()));
        $builder->leftJoin("attribute_group_lang","agl","agl.id_attribute_group = a.id_attribute_group AND al.id_lang =".cSQL($this->getDefaultEshopSetting()->getLanguageId()));
        $builder->where("pac.id_product_attribute = ".cSQL((int)$id_product_attribute));
        $builder->groupBy("pac.id_attribute");

        try {
            $result = Db::executeS($builder->build());
        } catch (Exception $exception) {
            throw new DataMiningFailedException($exception);
        }

        $dimensions = array();

        foreach($result as $row) {
            $dimension = new Dimension();
            $dimension->setDimensionId($row['id_attribute_group']);
            $dimension->setDimensionName($row['name']);
            $dimension->setDimensionValue($row['value']);
            $dimension->setDimensionValueId($row['id_attribute']);
            $dimensions[] = $dimension;
        }

        return $dimensions;
    }

    /**
     * @param DbQuery|null $builder
     * @return bool
     */
    protected function getDefaultSelectAttributes($builder = null) {
        $columns = array(
            "pa.id_product_attribute",
            "pa.ean13",
            "pa.reference",
            "pa.upc",
            "pa.quantity",
            "pa.id_product",
            "pas.weight",
            "IF(pas.wholesale_price != 0, pas.wholesale_price, ps.wholesale_price) AS purchase_price",
            "IF(pas.price != 0,  pas.price,  ps.price) AS price",
            "s.name AS distributor",
            "m.name AS brand",
            "ps.unity",
            "MIN(pai.id_image) AS id_image",
            "pl.link_rewrite",
            "pl.name",
            "ps.date_add",
            "ps.date_upd",
            "pl.id_shop"
        );
        $builder->select(implode(", ", $columns));
        return true;
    }

    /**
     * @param DbQuery|null $builder
     * @return bool
     */
    protected function getDefaultTable($builder = null) {
        $builder->from("product_attribute","pa");
        $builder->leftJoin("product_lang", "pl", "pa.id_product = pl.id_product AND pl.id_lang = '" . cSQL($this->getDefaultEshopSetting()->getLanguageId()) . "'");
        $builder->leftJoin("product_attribute_image", "pai", "pa.id_product_attribute = pai.id_product_attribute");
        $builder->leftJoin("product", "p", "pa.id_product = p.id_product");
        $builder->leftJoin("supplier", "s", "p.id_supplier = s.id_supplier");
        $builder->leftJoin("manufacturer", "m", "p.id_manufacturer = m.id_manufacturer");
        $builder->groupBy("pa.id_product_attribute");
        return true;
    }

    /**
     * @param DbQuery|null $builder
     * @return bool
     */
    protected function getDefaultCondition($builder = null) {
        $shopId = $this->getRequest()->getCurrentDefaultSetting("id_shop");

        if (!is_null($shopId)) {
            $builder->where("pl.id_shop = '" . cSQL($shopId->getValue()) . "'");
            $builder->innerJoin("product_shop", "ps", "p.id_product = ps.id_product AND ps.id_shop = '" . cSQL($shopId->getValue()) . "'");
            $builder->innerJoin("product_attribute_shop", "pas", "pa.id_product_attribute = pas.id_product_attribute AND pas.id_shop = '" . cSQL($shopId->getValue()) . "'");
        }

        if (!is_null($this->getRequest()->getId())) {
            $builder->where("pa.id_product_attribute = ".cSQL($this->getRequest()->getId()));
            return true;
        }
        if (!is_null($this->getRequest()->getForeignKey())) {
            $builder->where("pa.id_product = ".cSQL($this->getRequest()->getForeignKey()));
            return true;
        }

        //Select by dates
        $createdCondition = "";
        $updatedCondition = "";
        $dateCondition = "";
        $createdFrom = $this->getRequest()->getCreatedFrom();
        $createdTo = $this->getRequest()->getCreatedTo();
        $updatedFrom = $this->getRequest()->getUpdatedFrom();
        $updatedTo = $this->getRequest()->getUpdatedTo();

        if (!is_null($createdFrom) && !is_null($createdTo)) {
            $createdCondition .= "p.date_add BETWEEN '" . cSQL($createdFrom) . "' AND '" . cSQL($createdTo) . "'";
        } else if (!is_null($createdFrom)) {
            $createdCondition .= "p.date_add >= '" . cSQL($createdFrom) . "'";
        } else if (!is_null($createdTo)) {
            $createdCondition .= "p.date_add <= '" . cSQL($createdTo) . "'";
        }

        if (!is_null($updatedFrom) && !is_null($updatedTo)) {
            $updatedCondition .= "p.date_upd BETWEEN '" . cSQL($updatedFrom) . "' AND '" . cSQL($updatedTo) . "'";
        } else if (!is_null($updatedFrom)) {
            $updatedCondition .= "p.date_upd >= '" . cSQL($updatedFrom) . "'";
        } else if (!is_null($updatedTo)) {
            $updatedCondition .= "p.date_upd <= '" . cSQL($updatedTo) . "'";
        }

        if (!empty($createdCondition) && !empty($updatedCondition)) {
            $dateCondition = $createdCondition . " OR " . $updatedCondition;
        } else {
            $dateCondition = $createdCondition . $updatedCondition;
        }

        if (!empty($dateCondition)) {
            $builder->where($dateCondition);
            return true;
        }

        $sinceId = $this->getRequest()->getSinceId();
        $maxId = $this->getRequest()->getMaxId();

        if (!is_null($sinceId) && !is_null($maxId)) {
            $builder->where("pa.id_product_attribute BETWEEN ". cSQL($sinceId+1) . " AND " . cSQL($maxId-1));
            return true;
        } else if (!is_null($sinceId)) {
            $builder->where("pa.id_product_attribute > ". cSQL($sinceId));
            return true;
        } else if (!is_null($maxId)) {
            $builder->where("pa.id_product_attribute < ". cSQL($maxId));
            return true;
        }

        return false;
    }

    /**
     * @param DbQuery|null $builder
     * @return bool
     */
    protected function getDefaultLimits($builder = null) {
        $builder->limit($this->getResultsCount(), $this->getOffset());
        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentAttributes($builder = null) {
        $return = false;

        foreach ($this->getRequest()->getDifferences()->getColumns() as $column) {
            $return = true;
            $builder->select($column->__toString());
        }

        if ($return) return true;
        return null;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentTable($builder = null) {
        $return = false;

        foreach ($this->getRequest()->getDifferences()->getTableAliases() as $table) {
            $return = true;

            if ($table->hasConditions()) {
                $conditions = "";

                foreach ($table->getConditions() as $condition) {
                    $conditions .= $condition->__toString();
                }

                $conditions = $this->getRequest()->setupVariables($conditions);

                switch ($table->getType()) {
                    case Join::INNER:
                        $builder->innerJoin($table->getTableName(), $table->getAlias(), $conditions);
                        break;
                    case Join::LEFT:
                        $builder->leftJoin($table->getTableName(), $table->getAlias(), $conditions);
                        break;
                }
            } else {
                $builder->from($table->getTableName(), $table->getAlias());
            }
        }

        if ($return) return true;
        return null;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentConditions($builder = null) {
        $return = false;
        $conditions = "";

        foreach ($this->getRequest()->getDifferences()->getAdditionalConditions() as $condition) {
            $return = true;
            $conditions .= $condition->__toString();
        }

        $builder->where($this->getRequest()->setupVariables($conditions));

        if ($return) return true;
        return null;
    }

    /**
     * @return PrestaDefaultEshopSetting
     */
    protected function getDefaultEshopSetting() {
        return parent::getDefaultEshopSetting();
    }
}