<?php

namespace Monkey\PluginsPrestashop16XApi;


use Monkey\ImportEshopApi\Base\AImportEshopApi;
use Monkey\ImportEshopApi\Base\Input;
use Monkey\ImportEshopDataObjects\Base\Differences;
use Monkey\ImportEshopDataObjects\Entity\Simple\Alias;
use Monkey\ImportEshopDataObjects\Entity\Simple\Condition;
use Monkey\ImportEshopDataObjects\Entity\Simple\CurrentSetting;
use Monkey\ImportEshopDataObjects\Entity\Simple\Join;
use Monkey\ImportEshopDataObjects\Request\AttributeValuesRequest;
use Monkey\ImportEshopDataObjects\Request\BrandsRequest;
use Monkey\ImportEshopDataObjects\Request\CategoriesRequest;
use Monkey\ImportEshopDataObjects\Request\CouponsRequest;
use Monkey\ImportEshopDataObjects\Request\CustomersRequest;
use Monkey\ImportEshopDataObjects\Request\DistributorsRequest;
use Monkey\ImportEshopDataObjects\Request\LabelsRequest;
use Monkey\ImportEshopDataObjects\Request\OrderItemsRequest;
use Monkey\ImportEshopDataObjects\Request\OrdersRequest;
use Monkey\ImportEshopDataObjects\Request\OrderStatusesRequest;
use Monkey\ImportEshopDataObjects\Request\PaymentsRequest;
use Monkey\ImportEshopDataObjects\Request\ProductsRequest;
use Monkey\ImportEshopDataObjects\Request\ShipmentsRequest;
use Monkey\ImportEshopDataObjects\Request\TestRequest;
use Monkey\ImportEshopDataObjects\Request\VariantsRequest;
use Monkey\PluginsPrestashop16XApi\Base\PrestaDefaultEshopSetting;
use Monkey\PluginsPrestashop16XApi\Endpoint\AttributeValues;
use Monkey\PluginsPrestashop16XApi\Endpoint\Brands;
use Monkey\PluginsPrestashop16XApi\Endpoint\Categories;
use Monkey\PluginsPrestashop16XApi\Endpoint\Coupons;
use Monkey\PluginsPrestashop16XApi\Endpoint\Customers;
use Monkey\PluginsPrestashop16XApi\Endpoint\Distributors;
use Monkey\PluginsPrestashop16XApi\Endpoint\EshopInformation;
use Monkey\PluginsPrestashop16XApi\Endpoint\Labels;
use Monkey\PluginsPrestashop16XApi\Endpoint\OrderItems;
use Monkey\PluginsPrestashop16XApi\Endpoint\Orders;
use Monkey\PluginsPrestashop16XApi\Endpoint\OrderStatuses;
use Monkey\PluginsPrestashop16XApi\Endpoint\Payments;
use Monkey\PluginsPrestashop16XApi\Endpoint\Products;
use Monkey\PluginsPrestashop16XApi\Endpoint\Shipments;
use Monkey\PluginsPrestashop16XApi\Endpoint\Variants;
use Monkey\PluginsPrestashop16XApi\Facades\Configuration;
use Monkey\PluginsPrestashop16XApi\Facades\Db;
use NilPortugues\Serializer\JsonSerializer;

/**
 * Class Api
 * @package Monkey\PluginsPrestashop16XApi
 * @author Fingarfae
 */
class Api extends AImportEshopApi {

    const version = "Monkey\PluginsPrestashop16XApi\Api";

    public function __construct() {
        parent::__construct(new LocalhostSecurityHandler(), new JsonSerializer());
    }

    protected function handleEndpointInput() {
        return Input::get(self::PARAMETER_ENDPOINT);
    }

    protected function handleRequestInput() {
        //return Input::get(self::PARAMETER_REQUEST);

        if(is_null(Input::get(self::PARAMETER_REQUEST))){
            return $this->getMockRequest($this->handleEndpointInput());
        }
        return Input::get(self::PARAMETER_REQUEST);
    }

    protected function registerDefaultEshopSetting() {
        $setting = new PrestaDefaultEshopSetting(_DB_PREFIX_);

        $setting->setCurrencyId(Configuration::get('PS_CURRENCY_DEFAULT'));
        $setting->setCurrencyIso(Db::getValue("SELECT iso_code FROM "._DB_PREFIX_."currency WHERE id_currency = ".cSQL($setting->getCurrencyId())));

        $setting->setLanguageId(Configuration::get('PS_LANG_DEFAULT'));
        $setting->setLanguageIso(Db::getValue("SELECT iso_code FROM "._DB_PREFIX_."lang WHERE id_lang = ".cSQL($setting->getLanguageId())));

        $this->setDefaultEshopSetting($setting);
    }

    protected function registerEndpoints(){
        $this->registerEndpoint(new Orders($this->getFakeRequest(),$this->getFakeResponse(), $this->getDefaultEshopSetting()));
        $this->registerEndpoint(new OrderItems($this->getFakeRequest(),$this->getFakeResponse(), $this->getDefaultEshopSetting()));
        $this->registerEndpoint(new OrderStatuses($this->getFakeRequest(),$this->getFakeResponse(), $this->getDefaultEshopSetting()));

        $this->registerEndpoint(new Customers($this->getFakeRequest(),$this->getFakeResponse(), $this->getDefaultEshopSetting()));
        $this->registerEndpoint(new Payments($this->getFakeRequest(),$this->getFakeResponse(), $this->getDefaultEshopSetting()));
        $this->registerEndpoint(new Shipments($this->getFakeRequest(),$this->getFakeResponse(), $this->getDefaultEshopSetting()));

        $this->registerEndpoint(new Products($this->getFakeRequest(),$this->getFakeResponse(), $this->getDefaultEshopSetting()));
        $this->registerEndpoint(new Variants($this->getFakeRequest(),$this->getFakeResponse(), $this->getDefaultEshopSetting()));
        $this->registerEndpoint(new Categories($this->getFakeRequest(),$this->getFakeResponse(), $this->getDefaultEshopSetting()));

        $this->registerEndpoint(new Brands($this->getFakeRequest(),$this->getFakeResponse(), $this->getDefaultEshopSetting()));
        $this->registerEndpoint(new Distributors($this->getFakeRequest(),$this->getFakeResponse(), $this->getDefaultEshopSetting()));
        $this->registerEndpoint(new Labels($this->getFakeRequest(),$this->getFakeResponse(), $this->getDefaultEshopSetting()));
        $this->registerEndpoint(new Payments($this->getFakeRequest(),$this->getFakeResponse(), $this->getDefaultEshopSetting()));
        $this->registerEndpoint(new Products($this->getFakeRequest(),$this->getFakeResponse(), $this->getDefaultEshopSetting()));
        $this->registerEndpoint(new Shipments($this->getFakeRequest(),$this->getFakeResponse(), $this->getDefaultEshopSetting()));
        $this->registerEndpoint(new AttributeValues($this->getFakeRequest(),$this->getFakeResponse(), $this->getDefaultEshopSetting()));
        $this->registerEndpoint(new Coupons($this->getFakeRequest(),$this->getFakeResponse(), $this->getDefaultEshopSetting()));

        $this->registerEndpoint(new EshopInformation($this->getFakeRequest(),$this->getFakeResponse(), $this->getDefaultEshopSetting()));
    }

    /**
     * @param string $endpoint
     * @return string
     */
    public function getMockRequest($endpoint) {
        switch ($endpoint) {
            case "orders":
                $request = new OrdersRequest();
                //obecne
                $request->setHash(123);
                $request->setPage(1);
                $request->setLimit(100);
                /*
                $request->setCreatedFrom("2017-12-15 00:00:00");
                $request->setCreatedTo("2017-12-20 23:59:59");
                $request->setUpdatedFrom("2017-12-15 00:00:00");
                $request->setUpdatedTo("2017-12-20 23:59:59");
                */
                break;
            case "brands":
                $request = new BrandsRequest();
                //obecne
                $request->setHash(123);
                $request->setPage(1);
                $request->setLimit(100);
                break;
            case "distributors":
                $request = new DistributorsRequest();
                //obecne
                $request->setHash(123);
                $request->setPage(1);
                $request->setLimit(100);

                $request->setCreatedFrom("2017-12-01 00:00:00");
                $request->setCreatedTo("2017-12-31 23:59:59");
                $request->setUpdatedFrom("2017-12-01 00:00:00");
                $request->setUpdatedTo("2017-12-31 23:59:59");
                break;
            case "orderitems":
                $request = new OrderItemsRequest();
                //obecne
                $request->setHash(123);
                $request->setPage(1);
                $request->setLimit(100);

                $request->setForeignKey(96);
                break;
            case "orderstatuses":
                $request = new OrderStatusesRequest();
                $request->setHash(123);
                $request->setPage(1);
                $request->setLimit(100);
                break;
            case "categories":
                $request = new CategoriesRequest();
                $request->setHash(123);
                $request->setPage(1);
                $request->setLimit(100);
                break;
            case "labels":
                $request = new LabelsRequest();
                $request->setHash(123);
                $request->setPage(1);
                $request->setLimit(100);
                break;
            case "customers":
                $request = new CustomersRequest();
                $request->setHash(123);
                $request->setPage(1);
                $request->setLimit(100);
                break;
            case "payments":
                $request = new PaymentsRequest();
                $request->setHash(123);
                $request->setPage(1);
                $request->setLimit(100);
                break;
            case "shipments":
                $request = new ShipmentsRequest();
                $request->setHash(123);
                $request->setPage(1);
                $request->setLimit(100);
                break;
            case "attributevalues":
                $request = new AttributeValuesRequest();
                $request->setHash(123);
                $request->setPage(1);
                $request->setLimit(100);
                break;
            case "products":
                $request = new ProductsRequest();
                $request->setHash(123);
                $request->setPage(1);
                $request->setLimit(100);

                $request->setCreatedFrom("2017-12-01 00:00:00");
                $request->setCreatedTo("2017-12-31 23:59:59");
                $request->setUpdatedFrom("2017-12-01 00:00:00");
                $request->setUpdatedTo("2017-12-31 23:59:59");
                break;
            case "variants":
                $request = new VariantsRequest();
                $request->setHash(123);
                $request->setPage(1);
                $request->setLimit(10);

                $request->setCreatedFrom("2017-12-01 00:00:00");
                $request->setCreatedTo("2017-12-31 23:59:59");
                $request->setUpdatedFrom("2017-12-01 00:00:00");
                $request->setUpdatedTo("2017-12-31 23:59:59");
                break;
            case "coupons":
                $request = new CouponsRequest();
                $request->setHash(123);
                $request->setPage(1);
                $request->setLimit(2);
                $differences = new Differences();
                $differences->addTableAlias(new Join("cart_rule_lang", "coupon_lang", [new Condition(Condition::OPERATOR_WHERE, "coupon.id_cart_rule", Condition::COMPARE_OPERATOR_EQUALS, "coupon_lang.id_cart_rule")]));
                $differences->addColumn(new Alias("coupon_lang.name", "name"));
                $differences->addTableAlias(new Join("cart_rule", "coupon"));
                $differences->addColumn(new Alias("coupon.code", "code"));
                $differences->addColumn(new Alias("coupon.id_cart_rule", "id"));
                $request->setDifferences($differences);
                break;
            default:
                $request = new TestRequest();
                $request->setHash(123);
                $request->setPage(1);
                $request->setLimit(10);
        }

        $currentDefaultSettings = new CurrentSetting();
        $currentDefaultSettings->setKey("id_shop");
        $currentDefaultSettings->setValue(1);
        $request->addCurrentDefaultSettings($currentDefaultSettings);

        $currentDefaultSettings2 = new CurrentSetting();
        $currentDefaultSettings2->setKey("id_shop_group");
        $currentDefaultSettings2->setValue(1);
        $request->addCurrentDefaultSettings($currentDefaultSettings2);

        $request->setDebug(true);

        $serializer = new JsonSerializer();
        return $serializer->serialize($request);
    }
}