<?php
/**
 * Created by PhpStorm.
 * User: Fingarfae
 * Date: 31. 1. 2018
 * Time: 8:30
 */

namespace Monkey\PluginsPrestashop16XApi\Base;


use Monkey\ImportEshopApi\Exceptions\DataMiningFailedException;
use Monkey\ImportEshopDataObjects\Entity\Category;
use Monkey\PluginsPrestashop16XApi\Facades\Db;
use Monkey\PluginsPrestashop16XApi\Facades\DbQuery;

class CategoryHelper
{
    /**
     * @var CategoryData
     */
    private $data;

    /**
     * @var Category[]
     */
    private $result = array();

    /**
     * @var int
     */
    private $defaultLanguageId;

    /**
     * @var int
     */
    private $id_shop;

    /**
     * @var int
     */
    private $id_shop_group;

    public function __construct($id_lang,$id_shop,$id_shop_group){
        $this->setDefaultLanguageId($id_lang);
        $this->setIdShop($id_shop);
        $this->setIdShopGroup($id_shop_group);
    }

    /**
     * @param $productId
     * @return array
     * @throws DataMiningFailedException
     */
    public function getCategoryIds($productId){
        $this->setResult(array());
        $this->doStep($this->getData(), $productId);
        return $this->getResult();
    }

    /**
     * @param CategoryData $category
     * @param int $productId
     */
    private function doStep($category,$productId){

        if(empty($category->getChildren())){
            if(in_array($productId, $category->getProductIds())){
                $this->addResult($category->getEntity());
            }
            return;
        }

        $last = true;
        /**
         * @var CategoryData $child
         */
        foreach ($category->getChildren() as $child){
            if(in_array($productId, $child->getProductIds())){
                $last = false;
                $this->doStep($child, $productId);
            }
        }

        if($last and in_array($productId, $category->getProductIds()))$this->addResult($category->getEntity());
    }


    /**
     * @return CategoryData
     * @throws DataMiningFailedException
     */
    private function loadCategoryData(){

        $builder = new DbQuery();
        $builder->select("c.id_category,c.id_parent,cp.id_product,c.level_depth,cl.name");
        $builder->from("category","c");
        $builder->leftJoin("category_product","cp","c.id_category = cp.id_category");
        $builder->leftJoin("category_lang","cl","c.id_category = cl.id_category AND cl.id_lang = ".cSQL($this->getDefaultLanguageId()));
        $builder->where("cl.id_shop = ".cSQL($this->getIdShop()));
        $builder->orderBy("c.id_parent,c.id_category");

        try {
            $result = Db::executeS($builder->build());
        } catch (\Exception $exception) {
            throw new DataMiningFailedException($exception);
        }

        $tmp = array();

        foreach ($result as $row){
            if(array_key_exists($row['id_category'], $tmp)){
                if(!is_null($row['id_product'])){
                    $tmp[$row['id_category']]->addProductId($row['id_product']);
                }
            }else{
                $categoryData = new CategoryData();
                $categoryData->setCategoryId($row['id_category']);
                $categoryData->setParentId($row['id_parent']);
                $categoryData->setName($row['name']);
                $categoryData->setLevel($row['level_depth']);

                if(!is_null($row['id_product'])){
                    $categoryData->addProductId($row['id_product']);
                }
                $tmp[$row['id_category']] = $categoryData;
            }
        }

        $rootIndex = null;

        /**
         * @var  CategoryData[] $tmp
         * @var CategoryData $item
         */
        foreach ($tmp as $item){
            if(array_key_exists($item->getParentId(), $tmp)){
                $tmp[$item->getParentId()]->addChild($item);
            }else{
                $rootIndex = $item->getCategoryId();
            }
        }
        return $tmp[$rootIndex];
    }

    /**
     * @return CategoryData
     */
    private function getData(){
        if(is_null($this->data)){
            $this->data = $this->loadCategoryData();
        }
        return $this->data;
    }

    /**
     * @return Category[]
     */
    private function getResult(){
        return $this->result;
    }

    /**
     * @param Category[] $result
     * @return CategoryHelper
     */
    private function setResult($result){
        $this->result = $result;
        return $this;
    }

    /**
     * @param Category $result
     * @return CategoryHelper
     */
    private function addResult($result){
        $this->result[$result->getId()] = $result;
        return $this;
    }

    /**
     * @return int
     */
    public function getDefaultLanguageId(){
        return $this->defaultLanguageId;
    }

    /**
     * @param int $defaultLanguageId
     * @return CategoryHelper
     */
    public function setDefaultLanguageId($defaultLanguageId){
        $this->defaultLanguageId = $defaultLanguageId;
        return $this;
    }

    /**
     * @return int
     */
    public function getIdShop(){
        return $this->id_shop;
    }

    /**
     * @param int $id_shop
     * @return CategoryHelper
     */
    public function setIdShop($id_shop){
        $this->id_shop = $id_shop;
        return $this;
    }

    /**
     * @return int
     */
    public function getIdShopGroup(){
        return $this->id_shop_group;
    }

    /**
     * @param int $id_shop_group
     * @return CategoryHelper
     */
    public function setIdShopGroup($id_shop_group){
        $this->id_shop_group = $id_shop_group;
        return $this;
    }




}