<?php

namespace Monkey\PluginsPrestashop16XApi\Base;


use Exception;
use Monkey\ImportEshopApi\Exceptions\DataMiningFailedException;
use Monkey\PluginsPrestashop15XApi\Facades\Db;
use Monkey\PluginsPrestashop15XApi\Facades\DbQuery;

/**
 * Class CouponRestrictionHelper
 * @package Monkey\PluginsPrestashop16XApi\Base
 */
class CouponRestrictionHelper {
    /**
     * @var array $couponRestrictions
     */
    private $couponRestrictions = array();

    /**
     * @return array
     */
    private function getCouponRestrictions() {
        if (empty($this->couponRestrictions)) {
            $this->load();
        }

        return $this->couponRestrictions;
    }

    /**
     * @throws DataMiningFailedException
     */
    private function load() {
        $rBuilder = new DbQuery();
        $rBuilder->select("coupon_restriction_group.id_cart_rule AS coupon_id, coupon_restriction_item.id_item AS id, coupon_restriction.type");
        $rBuilder->from("cart_rule_product_rule_group", "coupon_restriction_group");
        $rBuilder->innerJoin("cart_rule_product_rule", "coupon_restriction", "coupon_restriction_group.id_product_rule_group = coupon_restriction.id_product_rule_group");
        $rBuilder->innerJoin("cart_rule_product_rule_value", "coupon_restriction_item", "coupon_restriction.id_product_rule = coupon_restriction_item.id_product_rule");
        $rBuilder->where("coupon_restriction.type = 'products' OR coupon_restriction.type = 'categories'");

        try {
            $restrictions = Db::executeS($rBuilder->build());
        } catch (Exception $exception) {
            throw new DataMiningFailedException($exception);
        }

        foreach ($restrictions as $restriction) {
            $this->couponRestrictions[$restriction["coupon_id"]][$restriction["type"]][] = $restriction["id"];
        }
    }

    /**
     * @param $couponId
     * @return array
     */
    public function getCategoriesRestrictions($couponId) {
        return (empty($this->getCouponRestrictions()[$couponId]["categories"]) ? array() : $this->getCouponRestrictions()[$couponId]["categories"]);
    }

    /**
     * @param $couponId
     * @return array
     */
    public function getProductsRestrictions($couponId) {
        return (empty($this->getCouponRestrictions()[$couponId]["products"]) ? array() : $this->getCouponRestrictions()[$couponId]["products"]);
    }
}