<?php

namespace Monkey\PluginsPrestashop16XApi\Endpoint;


use Exception;
use Monkey\ImportEshopApi\Endpoint\ADistributors;
use Monkey\ImportEshopApi\Exceptions\DataMiningFailedException;
use Monkey\ImportEshopDataObjects\Entity\Simple\Join;
use Monkey\PluginsPrestashop16XApi\Base\PrestaDefaultEshopSetting;
use Monkey\PluginsPrestashop16XApi\Facades\Db;
use Monkey\PluginsPrestashop16XApi\Facades\DbQuery;

/**
 * Class Distributors
 * @package Monkey\PluginsPrestashop16XApi\Endpoint
 */
class Distributors extends ADistributors {
    /**
     * @return string
     */
    protected function getComposedQuery() {
        $builder = new DbQuery();
        $this->getSelectAttributes($builder);
        $this->getTable($builder);
        $this->getConditions($builder);
        $this->getDefaultLimits($builder);
        $sql = $builder->build();
        return $sql;
    }

    /**
     * @return string
     * @throws DataMiningFailedException
     */
    protected function getTableSchema() {
        $tables = array("supplier");

        $results = array();
        foreach($tables as $tableName){
            $tableName = $this->getDefaultEshopSetting()->getDbPrefix() . $tableName;
            $sql = "SHOW COLUMNS FROM {$tableName}";

            try {
                $columns = Db::executeS($sql);
            } catch (Exception $exception) {
                throw new DataMiningFailedException($exception);
            }

            foreach($columns as &$row){
                $row['table']=$tableName;
            }

            $results = array_merge($results, $columns);
        }

        return $results;
    }

    /**
     * @throws DataMiningFailedException
     */
    protected function setupResponseData() {
        $sql = $this->getComposedQuery();

        if ($this->getRequest()->isDebug()) {
            $this->getResponse()->setQuery($sql);
            $this->getResponse()->setSchema($this->getTableSchema());
        }

        try {
            $results = Db::executeS($sql);
        } catch (Exception $exception) {
            throw new DataMiningFailedException($exception);
        }

        if ($results) {
            foreach ($results as $row) {
                $this->getResponse()->addNewEntity()
                    ->setId($row["id"])
                    ->setName($row["name"])
                    ->setCreatedAt($row["created_at"])
                    ->setUpdatedAt($row["updated_at"])
                    ->setIdShop($row['shop_id']);
            }
        }
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultSelectAttributes($builder = null) {
        $columns = array(
            "distributor.id_supplier AS id",
            "distributor.name",
            "distributor.date_add AS created_at",
            "distributor.date_upd AS updated_at",
            "distributor_shop.id_shop AS shop_id"
        );
        $builder->select(implode(", ", $columns));
        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultTable($builder = null) {
        $builder->from("supplier", "distributor");
        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultCondition($builder = null) {
        $shopId = $this->getRequest()->getCurrentDefaultSetting("id_shop");

        if (!is_null($shopId)) {
            $builder->innerJoin("supplier_shop", "distributor_shop", "distributor.id_supplier = distributor_shop.id_supplier AND distributor_shop.id_shop = '" . cSQL($shopId->getValue()) . "'");
        }

        if (!is_null($this->getRequest()->getId())) {
            $builder->where("distributor.id_supplier = " . cSQL($this->getRequest()->getId()));
            return true;
        }

        $createdCondition = "";
        $updatedCondition = "";
        $dateCondition = "";
        $createdFrom = $this->getRequest()->getCreatedFrom();
        $createdTo = $this->getRequest()->getCreatedTo();
        $updatedFrom = $this->getRequest()->getUpdatedFrom();
        $updatedTo = $this->getRequest()->getUpdatedTo();

        if (!is_null($createdFrom) && !is_null($createdTo)) {
            $createdCondition .= "distributor.date_add BETWEEN '" . cSQL($createdFrom) . "' AND '" . cSQL($createdTo) . "'";
        } else if (!is_null($createdFrom)) {
            $createdCondition .= "distributor.date_add >= '" . cSQL($createdFrom) . "'";
        } else if (!is_null($createdTo)) {
            $createdCondition .= "distributor.date_add <= '" . cSQL($createdTo) . "'";
        }

        if (!is_null($updatedFrom) && !is_null($updatedTo)) {
            $updatedCondition .= "distributor.date_upd BETWEEN '" . cSQL($updatedFrom) . "' AND '" . cSQL($updatedTo) . "'";
        } else if (!is_null($updatedFrom)) {
            $updatedCondition .= "distributor.date_upd >= '" . cSQL($updatedFrom) . "'";
        } else if (!is_null($updatedTo)) {
            $updatedCondition .= "distributor.date_upd <= '" . cSQL($updatedTo) . "'";
        }

        if (!empty($createdCondition) && !empty($updatedCondition)) {
            $dateCondition = $createdCondition . " OR " . $updatedCondition;
        } else {
            $dateCondition = $createdCondition . $updatedCondition;
        }

        if (!empty($dateCondition)) {
            $builder->where($dateCondition);
            return true;
        }

        return false;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultLimits($builder = null) {
        $builder->limit($this->getResultsCount(), $this->getOffset());
        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentAttributes($builder = null) {
        $return = false;

        foreach ($this->getRequest()->getDifferences()->getColumns() as $column) {
            $return = true;
            $builder->select($column->__toString());
        }

        if ($return) return true;
        return null;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentTable($builder = null) {
        $return = false;

        foreach ($this->getRequest()->getDifferences()->getTableAliases() as $table) {
            $return = true;

            if ($table->hasConditions()) {
                $conditions = "";

                foreach ($table->getConditions() as $condition) {
                    $conditions .= $condition->__toString();
                }

                $conditions = $this->getRequest()->setupVariables($conditions);

                switch ($table->getType()) {
                    case Join::INNER:
                        $builder->innerJoin($table->getTableName(), $table->getAlias(), $conditions);
                        break;
                    case Join::LEFT:
                        $builder->leftJoin($table->getTableName(), $table->getAlias(), $conditions);
                        break;
                }
            } else {
                $builder->from($table->getTableName(), $table->getAlias());
            }
        }

        if ($return) return true;
        return null;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentConditions($builder = null) {
        $return = false;
        $conditions = "";

        foreach ($this->getRequest()->getDifferences()->getAdditionalConditions() as $condition) {
            $return = true;
            $conditions .= $condition->__toString();
        }

        $builder->where($this->getRequest()->setupVariables($conditions));

        if ($return) return true;
        return null;
    }

    /**
     * @return PrestaDefaultEshopSetting
     */
    protected function getDefaultEshopSetting() {
        return parent::getDefaultEshopSetting();
    }
}