<?php
namespace Monkey\PluginsPrestashop16XApi\Endpoint;

use Exception;
use Monkey\ImportEshopApi\Endpoint\ALabels;
use Monkey\ImportEshopApi\Exceptions\DataMiningFailedException;
use Monkey\ImportEshopDataObjects\Entity\Simple\Join;
use Monkey\PluginsPrestashop16XApi\Base\PrestaDefaultEshopSetting;
use Monkey\PluginsPrestashop16XApi\Facades\Db;
use Monkey\PluginsPrestashop16XApi\Facades\DbQuery;

/**
 * Class Labels
 * @package Monkey\PluginsPrestashop16XApi\Endpoint
 */
class Labels extends ALabels {
    /**
     * @return string
     */
    protected function getComposedQuery() {
        $builder = new DbQuery();
        $this->getSelectAttributes($builder);
        $this->getTable($builder);
        $this->getConditions($builder);
        $this->getDefaultLimits($builder);
        $sql = $builder->build();
        return $sql;
    }

    /**
     * @return string
     * @throws DataMiningFailedException
     */
    protected function getTableSchema() {
        $tables = array("tag");

        $results = array();
        foreach($tables as $tableName){
            $tableName = $this->getDefaultEshopSetting()->getDbPrefix() . $tableName;
            $sql = "SHOW COLUMNS FROM {$tableName}";

            try {
                $columns = Db::executeS($sql);
            } catch (Exception $exception) {
                throw new DataMiningFailedException($exception);
            }

            foreach($columns as &$row){
                $row['table']=$tableName;
            }

            $results = array_merge($results, $columns);
        }

        return $results;
    }

    /**
     * @throws DataMiningFailedException
     */
    protected function setupResponseData() {
        $sql = $this->getComposedQuery();

        if ($this->getRequest()->isDebug()) {
            $this->getResponse()->setQuery($sql);
            $this->getResponse()->setSchema($this->getTableSchema());
        }

        try {
            $results = Db::executeS($sql);
        } catch (Exception $exception) {
            throw new DataMiningFailedException($exception);
        }

        if ($results) {
            foreach ($results as $row) {
                $this->getResponse()->addNewEntity()
                    ->setId($row["id"])
                    ->setName($row["name"]);
            }
        }
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultSelectAttributes($builder = null) {
        $columns = array(
            "label.id_tag AS id",
            "label.name"
        );
        $builder->select(implode(", ", $columns));
        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultTable($builder = null) {
        $builder->from("tag", "label");
        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultCondition($builder = null) {
        $langId = cSQL($this->getDefaultEshopSetting()->getLanguageId());
        $builder->where("label.id_lang = '{$langId}'");

        if (!is_null($this->getRequest()->getId())) {
            $builder->where("label.id_tag = '" . cSQL($this->getRequest()->getId()) . "'");
            return true;
        }

        $sinceId = $this->getRequest()->getSinceId();
        $maxId = $this->getRequest()->getMaxId();
        $idFilter = false;

        if (!is_null($sinceId)) {
            $builder->where("label.id_tag > " . cSQL($sinceId));
            $idFilter = true;
        }

        if (!is_null($maxId)) {
            $builder->where("label.id_tag < " . cSQL($maxId));
            $idFilter = true;
        }

        if ($idFilter) {
            return true;
        }

        return false;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultLimits($builder = null) {
        $builder->limit($this->getResultsCount(), $this->getOffset());
        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentAttributes($builder = null) {
        $return = false;

        foreach ($this->getRequest()->getDifferences()->getColumns() as $column) {
            $return = true;
            $builder->select($column->__toString());
        }

        if ($return) return true;
        return null;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentTable($builder = null) {
        $return = false;

        foreach ($this->getRequest()->getDifferences()->getTableAliases() as $table) {
            $return = true;

            if ($table->hasConditions()) {
                $conditions = "";

                foreach ($table->getConditions() as $condition) {
                    $conditions .= $condition->__toString();
                }

                $conditions = $this->getRequest()->setupVariables($conditions);

                switch ($table->getType()) {
                    case Join::INNER:
                        $builder->innerJoin($table->getTableName(), $table->getAlias(), $conditions);
                        break;
                    case Join::LEFT:
                        $builder->leftJoin($table->getTableName(), $table->getAlias(), $conditions);
                        break;
                }
            } else {
                $builder->from($table->getTableName(), $table->getAlias());
            }
        }

        if ($return) return true;
        return null;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentConditions($builder = null) {
        $return = false;
        $conditions = "";

        foreach ($this->getRequest()->getDifferences()->getAdditionalConditions() as $condition) {
            $return = true;
            $conditions .= $condition->__toString();
        }

        $builder->where($this->getRequest()->setupVariables($conditions));

        if ($return) return true;
        return null;
    }

    /**
     * @return PrestaDefaultEshopSetting
     */
    protected function getDefaultEshopSetting() {
        return parent::getDefaultEshopSetting();
    }
}