<?php
/**
 * Created by PhpStorm.
 * User: Fingarfae
 * Date: 6. 2. 2018
 * Time: 14:15
 */

namespace Monkey\PluginsPrestashop17XApi\Endpoint;


use Monkey\ImportEshopApi\Endpoint\AEshopInformation;
use Monkey\ImportEshopApi\Exceptions\DataMiningFailedException;
use Monkey\ImportEshopDataObjects\Entity\Simple\Join;
use Monkey\PluginsPrestashop17XApi\Base\PrestaDefaultEshopSetting;
use Monkey\PluginsPrestashop17XApi\Facades\Db;
use Monkey\PluginsPrestashop17XApi\Facades\DbQuery;
use Exception;

class EshopInformation extends AEshopInformation
{
    /**
     * @return string
     */
    protected function getComposedQuery() {
        $builder = new DbQuery();
        $this->getSelectAttributes($builder);
        $this->getTable($builder);
        $this->getConditions($builder);
        $this->getDefaultLimits($builder);
        $sql = $builder->build();
        return $sql;
    }

    /**
     * @return string
     * @throws DataMiningFailedException
     */
    protected function getTableSchema() {
        $tables = array("shop");

        $results = array();
        foreach($tables as $tableName){
            $tableName = $this->getDefaultEshopSetting()->getDbPrefix() . $tableName;
            $sql = "SHOW COLUMNS FROM {$tableName}";

            try {
                $columns = Db::executeS($sql);
            } catch (Exception $exception) {
                throw new DataMiningFailedException($exception);
            }

            foreach($columns as &$row){
                $row['table']=$tableName;
            }

            $results = array_merge($results, $columns);
        }

        return $results;
    }

    /**
     * @throws DataMiningFailedException
     */
    protected function setupResponseData() {

        $sql = $this->getComposedQuery();

        if ($this->getRequest()->isDebug()) {
            $this->getResponse()->setQuery($sql);
            $this->getResponse()->setSchema($this->getTableSchema());
        }

        try {
            $results = Db::executeS($sql);
        } catch (Exception $exception) {
            throw new DataMiningFailedException($exception);
        }


        if ($results) {
            
            foreach ($results as $shop){
                $shop['id_language'] = $this->getDefaultEshopSetting()->getLanguageId();
                $shop['language_iso'] = $this->getDefaultEshopSetting()->getLanguageIso();
                $shop['id_currency'] = $this->getDefaultEshopSetting()->getCurrencyId();
                $shop['currency_iso'] = $this->getDefaultEshopSetting()->getCurrencyIso();
                $this->getResponse()->addNewEntity()
                    ->setData($shop);
            }

        }
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultSelectAttributes($builder = null) {
        $columns = array(
            "id_shop"," id_shop_group"," name"
        );
        $builder->select(implode(", ", $columns));
        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultTable($builder = null) {
        $builder->from("shop");
        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultCondition($builder = null) {


        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultLimits($builder = null) {
        $builder->limit($this->getResultsCount(), $this->getOffset());
        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentAttributes($builder = null) {
        $return = false;

        foreach ($this->getRequest()->getDifferences()->getColumns() as $column) {
            $return = true;
            $builder->select($column->__toString());
        }

        if ($return) return true;
        return null;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentTable($builder = null) {
        $return = false;

        foreach ($this->getRequest()->getDifferences()->getTableAliases() as $table) {
            $return = true;

            if ($table->hasConditions()) {
                $conditions = "";

                foreach ($table->getConditions() as $condition) {
                    $conditions .= $condition->__toString();
                }

                $conditions = $this->getRequest()->setupVariables($conditions);

                switch ($table->getType()) {
                    case Join::INNER:
                        $builder->innerJoin($table->getTableName(), $table->getAlias(), $conditions);
                        break;
                    case Join::LEFT:
                        $builder->leftJoin($table->getTableName(), $table->getAlias(), $conditions);
                        break;
                }
            } else {
                $builder->from($table->getTableName(), $table->getAlias());
            }
        }

        if ($return) return true;
        return null;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentConditions($builder = null) {
        $return = false;
        $conditions = "";

        foreach ($this->getRequest()->getDifferences()->getAdditionalConditions() as $condition) {
            $return = true;
            $conditions .= $condition->__toString();
        }

        $builder->where($this->getRequest()->setupVariables($conditions));

        if ($return) return true;
        return null;
    }

    /**
     * @return PrestaDefaultEshopSetting
     */
    protected function getDefaultEshopSetting() {
        return parent::getDefaultEshopSetting();
    }
}