<?php
/**
 * Created by PhpStorm.
 * User: Fingarfae
 * Date: 23. 1. 2018
 * Time: 11:22
 */

namespace Monkey\PluginsPrestashop17XApi\Endpoint;


use Monkey\ImportEshopApi\Endpoint\AProducts;
use Monkey\ImportEshopApi\Exceptions\DataMiningFailedException;
use Monkey\ImportEshopDataObjects\Entity\Simple\Join;
use Monkey\PluginsPrestashop17XApi\Base\CategoryHelper;
use Monkey\PluginsPrestashop17XApi\Base\LabelHelper;
use Monkey\PluginsPrestashop17XApi\Base\PrestaDefaultEshopSetting;
use Monkey\PluginsPrestashop17XApi\Facades\Db;
use Monkey\PluginsPrestashop17XApi\Facades\DbQuery;
use Monkey\PluginsPrestashop17XApi\Facades\Link;
use Exception;

class Products extends  AProducts
{

    /**
     * @return string
     */
    protected function getComposedQuery() {
        $builder = new DbQuery();
        $this->getSelectAttributes($builder);
        $this->getTable($builder);
        $this->getConditions($builder);
        $this->getDefaultLimits($builder);
        $sql = $builder->build();
        return $sql;
    }

    /**
     * @return string
     * @throws DataMiningFailedException
     */
    protected function getTableSchema() {

        $tables = array("product","product_lang","image");

        $results = array();
        foreach($tables as $tableName){
            $tableName = $this->getDefaultEshopSetting()->getDbPrefix() . $tableName;
            $sql = "SHOW COLUMNS FROM {$tableName}";

            try {
                $columns = Db::executeS($sql);
            } catch (Exception $exception) {
                throw new DataMiningFailedException($exception);
            }

            foreach($columns as &$row){
                $row['table']=$tableName;
            }

            $results = array_merge($results, $columns);
        }

        return $results;
    }

    /**
     * @throws DataMiningFailedException
     */
    protected function setupResponseData() {

        $sql = $this->getComposedQuery();

        try {
            $results = Db::executeS($sql);
        } catch (Exception $exception) {
            throw new DataMiningFailedException($exception);
        }

        if ($results) {

            $categoryHelper = new CategoryHelper(
                $this->getDefaultEshopSetting()->getLanguageId(),
                $this->getRequest()->getCurrentDefaultSetting("id_shop")->getValue(),
                $this->getRequest()->getCurrentDefaultSetting("id_shop_group")->getValue()
                );

            $labelHelper = new LabelHelper($this->getDefaultEshopSetting()->getLanguageId());

            foreach ($results as $row) {

                $product = $this->getResponse()->addNewEntity();
                $product->setId($row['id_product']);
                $product->setIdShop($row['id_shop']);
                $product->setCreatedAt($row['date_add']);
                $product->setUpdatedAt($row['date_upd']);

                $product->setName($row['name']);
                $product->setDescription(strip_tags($row['description']));
                $product->setBrandId($row['brand_id']);
                $product->setImageThumb(!is_null($row['id_image'])?Link::getImageLink($row['link_rewrite'], $row['id_image'], Link::IMAGE_TYPE_SMALL):"");
                $product->setImageSrc(!is_null($row['id_image'])?Link::getImageLink($row['link_rewrite'], $row['id_image'], Link::IMAGE_TYPE_LARGE):"");

                $product->setCategories($categoryHelper->getCategoryIds($row['id_product']));
                $product->setLabels($labelHelper->getLabels($row['id_product']));

            }
        }
    }

    /**
     * @param DbQuery|null $builder
     * @return bool
     */
    protected function getDefaultSelectAttributes($builder = null) {
        $columns = array(
            "p.id_product",
            "p.id_manufacturer as brand_id",
            "pl.description",
            "pl.name",
            "ps.date_add",
            "ps.date_upd",
            "pl.link_rewrite",
            "i.id_image",
            "pl.id_shop"
        );
        $builder->select(implode(", ", $columns));
        return true;
    }

    /**
     * @param DbQuery|null $builder
     * @return bool
     */
    protected function getDefaultTable($builder = null) {
        $builder->from("product","p");
        $builder->leftJoin("product_lang", "pl", "p.id_product = pl.id_product");
        $builder->leftJoin("image","i","p.id_product = i.id_product AND i.cover = 1");
        $builder->leftJoin("image_shop", "is", "i.id_image = is.id_image AND is.cover = 1");
        $builder->leftJoin("image_lang", "il", "i.id_image = il.id_image");
        $shopId = $this->getRequest()->getCurrentDefaultSetting("id_shop");
        if (!is_null($shopId)) {
            $builder->innerJoin("product_shop", "ps", "p.id_product = ps.id_product AND ps.id_shop = '" . cSQL($shopId->getValue()) . "'");
        }
        return true;
    }

    /**
     * @param DbQuery|null $builder
     * @return bool
     */
    protected function getDefaultCondition($builder = null) {
        $langId = cSQL($this->getDefaultEshopSetting()->getLanguageId());
        $builder->where("pl.id_lang = '{$langId}'");
        $builder->where("il.id_lang = '{$langId}'");

        $shopId = $this->getRequest()->getCurrentDefaultSetting("id_shop");
        if (!is_null($shopId)) {
            $builder->where("pl.id_shop = '" . cSQL($shopId->getValue()) . "'");
            $builder->where("is.id_shop = '" . cSQL($shopId->getValue()) . "'");
        }

        if (!is_null($this->getRequest()->getId())) {
            $builder->where("p.id_product = ".cSQL($this->getRequest()->getId()));
            return true;
        }

        //Select by dates
        $createdCondition = "";
        $updatedCondition = "";
        $dateCondition = "";
        $createdFrom = $this->getRequest()->getCreatedFrom();
        $createdTo = $this->getRequest()->getCreatedTo();
        $updatedFrom = $this->getRequest()->getUpdatedFrom();
        $updatedTo = $this->getRequest()->getUpdatedTo();

        if (!is_null($createdFrom) && !is_null($createdTo)) {
            $createdCondition .= "p.date_add BETWEEN '" . cSQL($createdFrom) . "' AND '" . cSQL($createdTo) . "'";
        } else if (!is_null($createdFrom)) {
            $createdCondition .= "p.date_add >= '" . cSQL($createdFrom) . "'";
        } else if (!is_null($createdTo)) {
            $createdCondition .= "p.date_add <= '" . cSQL($createdTo) . "'";
        }

        if (!is_null($updatedFrom) && !is_null($updatedTo)) {
            $updatedCondition .= "p.date_upd BETWEEN '" . cSQL($updatedFrom) . "' AND '" . cSQL($updatedTo) . "'";
        } else if (!is_null($updatedFrom)) {
            $updatedCondition .= "p.date_upd >= '" . cSQL($updatedFrom) . "'";
        } else if (!is_null($updatedTo)) {
            $updatedCondition .= "p.date_upd <= '" . cSQL($updatedTo) . "'";
        }
        if (!empty($createdCondition) && !empty($updatedCondition)) {
            $dateCondition = $createdCondition . " OR " . $updatedCondition;
        } else {
            $dateCondition = $createdCondition . $updatedCondition;
        }

        if (!empty($dateCondition)) {
            $builder->where($dateCondition);
            return true;
        }


        $sinceId = $this->getRequest()->getSinceId();
        $maxId = $this->getRequest()->getMaxId();
        if (!is_null($sinceId) && !is_null($maxId)) {
            $builder->where("p.id_product BETWEEN ". cSQL($sinceId+1) . " AND " . cSQL($maxId-1));
            return true;
        } else if (!is_null($sinceId)) {
            $builder->where("p.id_product > ". cSQL($sinceId));
            return true;
        } else if (!is_null($maxId)) {
            $builder->where("p.id_product < ". cSQL($maxId));
            return true;
        }
        return false;
    }

    /**
     * @param DbQuery|null $builder
     * @return bool
     */
    protected function getDefaultLimits($builder = null) {
        $builder->limit($this->getResultsCount(), $this->getOffset());
        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentAttributes($builder = null) {
        $return = false;

        foreach ($this->getRequest()->getDifferences()->getColumns() as $column) {
            $return = true;
            $builder->select($column->__toString());
        }

        if ($return) return true;
        return null;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentTable($builder = null) {
        $return = false;

        foreach ($this->getRequest()->getDifferences()->getTableAliases() as $table) {
            $return = true;

            if ($table->hasConditions()) {
                $conditions = "";

                foreach ($table->getConditions() as $condition) {
                    $conditions .= $condition->__toString();
                }

                $conditions = $this->getRequest()->setupVariables($conditions);

                switch ($table->getType()) {
                    case Join::INNER:
                        $builder->innerJoin($table->getTableName(), $table->getAlias(), $conditions);
                        break;
                    case Join::LEFT:
                        $builder->leftJoin($table->getTableName(), $table->getAlias(), $conditions);
                        break;
                }
            } else {
                $builder->from($table->getTableName(), $table->getAlias());
            }
        }

        if ($return) return true;
        return null;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentConditions($builder = null) {
        $return = false;
        $conditions = "";

        foreach ($this->getRequest()->getDifferences()->getAdditionalConditions() as $condition) {
            $return = true;
            $conditions .= $condition->__toString();
        }

        $builder->where($this->getRequest()->setupVariables($conditions));

        if ($return) return true;
        return null;
    }

    /**
     * @return PrestaDefaultEshopSetting
     */
    protected function getDefaultEshopSetting() {
        return parent::getDefaultEshopSetting();
    }
}