<?php

namespace Monkey\PluginsPrestashop17XApi\Endpoint;


use Exception;
use Monkey\ImportEshopApi\Endpoint\AShipments;
use Monkey\ImportEshopApi\Exceptions\DataMiningFailedException;
use Monkey\ImportEshopDataObjects\Entity\Simple\Join;
use Monkey\PluginsPrestashop17XApi\Base\PrestaDefaultEshopSetting;
use Monkey\PluginsPrestashop17XApi\Facades\Db;
use Monkey\PluginsPrestashop17XApi\Facades\DbQuery;

/**
 * Class Shipments
 * @package Monkey\PluginsPrestashop17XApi\Endpoint
 */
class Shipments extends AShipments {
    /**
     * @return string
     */
    protected function getComposedQuery() {
        $builder = new DbQuery();
        $this->getSelectAttributes($builder);
        $this->getTable($builder);
        $this->getConditions($builder);
        $this->getDefaultLimits($builder);
        $sql = $builder->build();
        return $sql;
    }

    /**
     * @return string
     * @throws DataMiningFailedException
     */
    protected function getTableSchema() {
        $tables = array("carrier");

        $results = array();
        foreach($tables as $tableName){
            $tableName = $this->getDefaultEshopSetting()->getDbPrefix() . $tableName;
            $sql = "SHOW COLUMNS FROM {$tableName}";

            try {
                $columns = Db::executeS($sql);
            } catch (Exception $exception) {
                throw new DataMiningFailedException($exception);
            }

            foreach($columns as &$row){
                $row['table']=$tableName;
            }

            $results = array_merge($results, $columns);
        }

        return $results;
    }

    /**
     * @throws DataMiningFailedException
     */
    protected function setupResponseData() {
        $sql = $this->getComposedQuery();

        if ($this->getRequest()->isDebug()) {
            $this->getResponse()->setQuery($sql);
            $this->getResponse()->setSchema($this->getTableSchema());
        }

        try {
            $results = Db::executeS($sql);
        } catch (Exception $exception) {
            throw new DataMiningFailedException($exception);
        }

        if ($results) {
            foreach ($results as $row) {
                $this->getResponse()->addNewEntity()
                    ->setId($row["id"])
                    ->setName($row["name"]);
            }
        }
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultSelectAttributes($builder = null) {
        $columns = array(
            "shipment.id_carrier AS id",
            "shipment.name AS name"
        );
        $builder->select(implode(", ", $columns));
        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultTable($builder = null) {
        $builder->from("carrier", "shipment");
        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultCondition($builder = null) {
        $shopId = $this->getRequest()->getCurrentDefaultSetting("id_shop");

        if (!is_null($shopId)) {
            $builder->innerJoin("carrier_shop", "shipment_shop", "shipment.id_carrier = shipment_shop.id_carrier AND shipment_shop.id_shop = '" . cSQL($shopId->getValue()) . "'");
        }

        if (!is_null($this->getRequest()->getId())) {
            $builder->where("shipment.id_carrier = '" . cSQL($this->getRequest()->getId()) . "'");
            return true;
        }

        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDefaultLimits($builder = null) {
        $builder->limit($this->getResultsCount(), $this->getOffset());
        return true;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentAttributes($builder = null) {
        $return = false;

        foreach ($this->getRequest()->getDifferences()->getColumns() as $column) {
            $return = true;
            $builder->select($column->__toString());
        }

        if ($return) return true;
        return null;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentTable($builder = null) {
        $return = false;

        foreach ($this->getRequest()->getDifferences()->getTableAliases() as $table) {
            $return = true;

            if ($table->hasConditions()) {
                $conditions = "";

                foreach ($table->getConditions() as $condition) {
                    $conditions .= $condition->__toString();
                }

                $conditions = $this->getRequest()->setupVariables($conditions);

                switch ($table->getType()) {
                    case Join::INNER:
                        $builder->innerJoin($table->getTableName(), $table->getAlias(), $conditions);
                        break;
                    case Join::LEFT:
                        $builder->leftJoin($table->getTableName(), $table->getAlias(), $conditions);
                        break;
                }
            } else {
                $builder->from($table->getTableName(), $table->getAlias());
            }
        }

        if ($return) return true;
        return null;
    }

    /**
     * @param mixed|null $builder
     * @return mixed|null
     */
    protected function getDifferentConditions($builder = null) {
        $return = false;
        $conditions = "";

        foreach ($this->getRequest()->getDifferences()->getAdditionalConditions() as $condition) {
            $return = true;
            $conditions .= $condition->__toString();
        }

        $builder->where($this->getRequest()->setupVariables($conditions));

        if ($return) return true;
        return null;
    }

    /**
     * @return PrestaDefaultEshopSetting
     */
    protected function getDefaultEshopSetting() {
        return parent::getDefaultEshopSetting();
    }
}