<?php

namespace Monkey\PluginsPrestashop17XApi\Facades;

/**
 * Class DbQuery
 * @package Monkey\PluginsPrestashop17XApi\Facades
 */
class DbQuery {
    /**
     * @var \DbQuery $dbQuery
     */
    private $dbQuery;

    /**
     * Sets type of the query
     *
     * @param string $type SELECT|DELETE
     *
     * @return DbQuery
     */
    public function type($type){
        $this->getDbQuery()->type($type);
        return $this;
    }

    /**
     * @param string $table
     * @param string|null $alias
     * @return DbQuery
     */
    public function from($table, $alias = null) {
        $this->getDbQuery()->from($table, $alias);
        return $this;
    }

    /**
     * @param string $table
     * @param string|null $alias
     * @param string|null $on
     * @return DbQuery
     */
    public function leftJoin($table, $alias = null, $on = null){
        $this->getDbQuery()->leftJoin($table, $alias, $on);
        return $this;
    }

    /**
     * @param string $table
     * @param string|null $alias
     * @param string|null $on
     * @return DbQuery
     */
    public function innerJoin($table, $alias = null, $on = null) {
        $this->getDbQuery()->innerJoin($table, $alias, $on);
        return $this;
    }

    /**
     * @param string $subQuery
     * @param string|null $alias
     * @param string|null $on
     * @return DbQuery
     */
    public function joinSubQuery($subQuery, $alias = null, $on = null) {
        $string = 'JOIN '.$subQuery.''.($alias ? ' `'.pSQL($alias).'`' : '').($on ? ' ON '.$on : '');
        $this->getDbQuery()->join($string);
        return $this;
    }

    /**
     * @param string $fields
     * @return DbQuery
     */
    public function select($fields) {
        $this->getDbQuery()->select($fields);
        return $this;
    }

    /**
     * @param string $restriction
     * @return DbQuery
     */
    public function where($restriction) {
        $this->getDbQuery()->where($restriction);
        return $this;
    }

    /**
     * @param string $fields
     * @return DbQuery
     */
    public function groupBy($fields) {
        $this->getDbQuery()->groupBy($fields);
        return $this;
    }

    /**
     * @param string $fields
     * @return DbQuery
     */
    public function orderBy($fields) {
        $this->getDbQuery()->orderBy($fields);
        return $this;
    }

    /**
     * @param int $limit
     * @param int $offset
     * @return DbQuery
     */
    public function limit($limit, $offset) {
        $this->getDbQuery()->limit($limit, $offset);
        return $this;
    }

    /**
     * @return string
     */
    public function build() {
        return $this->getDbQuery()->build();
    }

    /**
     * @return \DbQuery
     */
    public function getDbQuery() {
        if (is_null($this->dbQuery)) {
            $this->setDbQuery(new \DbQuery());
        }

        return $this->dbQuery;
    }

    /**
     * @param \DbQuery $dbQuery
     * @return DbQuery
     */
    protected function setDbQuery($dbQuery) {
        $this->dbQuery = $dbQuery;
        return $this;
    }
}