<?php

/**
 * Author: Nil Portugués Calderó <contact@nilportugues.com>
 * Date: 8/30/15
 * Time: 12:33 PM.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace NilPortugues\Test\Serializer;

use DateTime;
use NilPortugues\Serializer\JsonSerializer;
use NilPortugues\Test\Serializer\Dummy\ComplexObject\Comment;
use NilPortugues\Test\Serializer\Dummy\ComplexObject\Post;
use NilPortugues\Test\Serializer\Dummy\ComplexObject\User;
use NilPortugues\Test\Serializer\Dummy\ComplexObject\ValueObject\CommentId;
use NilPortugues\Test\Serializer\Dummy\ComplexObject\ValueObject\PostId;
use NilPortugues\Test\Serializer\Dummy\ComplexObject\ValueObject\UserId;

class JsonSerializerTest extends \PHPUnit_Framework_TestCase
{
    public function testSerialization()
    {
        $object = $this->getObject();
        $serializer = new JsonSerializer();
        $serializedObject = $serializer->serialize($object);

        $this->assertEquals($object, $serializer->unserialize($serializedObject));
    }

    private function getObject()
    {
        $created = new DateTime('2015/07/18 12:13:00');
        $accepted = new DateTime('2015/07/19 00:00:00');
        return new Post(
            new PostId(9),
            'Hello World',
            'Your first post',
            new User(
                new UserId(1),
                'Post Author'
            ),
            array(
                new Comment(
                    new CommentId(1000),
                    'Have no fear, sers, your king is safe.',
                    new User(new UserId(2), 'Barristan Selmy'),
                    array(
                        'created_at' => $created->format('c'),
                        'accepted_at' => $accepted->format('c'),
                    )
                ),
            )
        );
    }

    public function testArraySerialization()
    {
        $arrayOfObjects = array($this->getObject(), $this->getObject());
        $serializer = new JsonSerializer();
        $serializedObject = $serializer->serialize($arrayOfObjects);

        $this->assertEquals($arrayOfObjects, $serializer->unserialize($serializedObject));
    }
}
